@@ -23,12 +23,11 @@ usage() {
     echo "            [-a i386 | **amd64** | armhf ]"
     echo "            [-b **haproxy** ]"
     echo "            [-c **~/.cache/image-create** | <cache directory> ]"
-    echo "            [-d **trusty** | xenial | <other release id> ]"
     echo "            [-h]"
-    echo "            [-i **ubuntu** | fedora | centos ]"
+    echo "            [-i **ubuntu** | fedora | centos | rhel ]"
     echo "            [-o **amphora-x64-haproxy** | <filename> ]"
     echo "            [-r <root password> ]"
-    echo "            [-s **5** | <size in GB> ]"
+    echo "            [-s **2** | <size in GB> ]"
     echo "            [-t **qcow2** | tar | vhd ]"
     echo "            [-v]"
     echo "            [-w <working directory> ]"
@@ -36,12 +35,11 @@ usage() {
     echo "        '-a' is the architecture type for the image (default: amd64)"
     echo "        '-b' is the backend type (default: haproxy)"
     echo "        '-c' is the path to the cache directory (default: ~/.cache/image-create)"
-    echo "        '-d' distribution release id (default: trusty)"
     echo "        '-h' display this help message"
     echo "        '-i' is the base OS (default: ubuntu)"
     echo "        '-o' is the output image file name"
     echo "        '-r' enable the root account in the generated image (default: disabled)"
-    echo "        '-s' is the image size to produce in gigabytes (default: 5)"
+    echo "        '-s' is the image size to produce in gigabytes (default: 2)"
     echo "        '-t' is the image type (default: qcow2)"
     echo "        '-v' display the script version"
     echo "        '-w' working directory for image building (default: .)"
@@ -55,13 +53,24 @@ version() {
     exit 1
 }
 
+find_system_elements() {
+    # List of possible system installation directories
+    local system_prefixes="/usr/share /usr/local/share"
+    for prefix in $system_prefixes; do
+        if [ -d $prefix/$1 ]; then
+            echo $prefix/$1
+            return
+        fi
+    done
+}
+
 # Figure out where our directory is located
 if [ -z $OCTAVIA_REPO_PATH ]; then
     AMP_DIR=$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )
     OCTAVIA_REPO_PATH=${OCTAVIA_REPO_PATH:-${AMP_DIR%/*}}
 fi
 
-while getopts "a:b:c:d:hi:o:t:r:s:vw:" opt; do
+while getopts "a:b:c:hi:o:t:r:s:vw:" opt; do
     case $opt in
         a)
             AMP_ARCH=$OPTARG
@@ -83,9 +92,6 @@ while getopts "a:b:c:d:hi:o:t:r:s:vw:" opt; do
         c)
             AMP_CACHEDIR=$OPTARG
         ;;
-        d)
-            AMP_DIB_RELEASE=$OPTARG
-        ;;
         h)
             usage
         ;;
@@ -93,13 +99,14 @@ while getopts "a:b:c:d:hi:o:t:r:s:vw:" opt; do
             AMP_BASEOS=$OPTARG
             if [ $AMP_BASEOS != "ubuntu" ] && \
                 [ $AMP_BASEOS != "fedora" ] && \
-                [ $AMP_BASEOS != "centos" ]; then
+                [ $AMP_BASEOS != "centos" ] && \
+                [ $AMP_BASEOS != "rhel" ]; then
                 echo "Error: Unsupported base OS " $AMP_BASEOS " specified"
                 exit 3
             fi
         ;;
         o)
-            AMP_OUTPUTFILENAME=$OPTARG
+            AMP_OUTPUTFILENAME=$(readlink -f $OPTARG)
         ;;
         t)
             AMP_IMAGETYPE=$OPTARG
@@ -146,51 +153,46 @@ AMP_CACHEDIR=${AMP_CACHEDIR:-"$HOME/.cache/image-create"}
 
 AMP_BASEOS=${AMP_BASEOS:-"ubuntu"}
 
-export DIB_RELEASE=${AMP_DIB_RELEASE:-"trusty"}
-
-AMP_OUTPUTFILENAME=${AMP_OUTPUTFILENAME:-"$AMP_DIR/amphora-x64-haproxy"}
+AMP_OUTPUTFILENAME=${AMP_OUTPUTFILENAME:-"$PWD/amphora-x64-haproxy"}
 
 AMP_IMAGETYPE=${AMP_IMAGETYPE:-"qcow2"}
 
-AMP_IMAGESIZE=${AMP_IMAGESIZE:-5}
+AMP_IMAGESIZE=${AMP_IMAGESIZE:-2}
 
 OCTAVIA_ELEMENTS_PATH=$OCTAVIA_REPO_PATH/elements
 
-
-
 if ! [ -d $OCTAVIA_ELEMENTS_PATH ]; then
-    echo "ERROR: Octavia elements directory not found at: " $OCTAVIA_ELEMENTS_PATH " Exiting."
-    exit 1
+    SYSTEM_OCTAVIA_ELEMENTS_PATH=$(find_system_elements octavia-image-elements)
+    if [ -z ${SYSTEM_OCTAVIA_ELEMENTS_PATH} ]; then
+        echo "ERROR: Octavia elements directory not found at: " $OCTAVIA_ELEMENTS_PATH " Exiting."
+        exit 1
+    fi
+    OCTAVIA_ELEMENTS_PATH=${SYSTEM_OCTAVIA_ELEMENTS_PATH}
 fi
 
 DIB_REPO_PATH=${DIB_REPO_PATH:-${OCTAVIA_REPO_PATH%/*}/diskimage-builder}
 
-if ! [ -d $DIB_REPO_PATH ]; then
-    echo "ERROR: diskimage-builder repo directory not found at: " $DIB_REPO_PATH " Exiting."
-    exit 1
+if [ -d $DIB_REPO_PATH ]; then
+    export PATH=$PATH:$DIB_REPO_PATH/bin
+else
+    if ! disk-image-create --version > /dev/null 2>&1; then
+        echo "ERROR: diskimage-builder repo directory not found at: " $DIB_REPO_PATH " or in path. Exiting."
+        exit 1
+    fi
 fi
 
-export PATH=$PATH:$DIB_REPO_PATH/bin
-
+# For system-wide installs, DIB will automatically find the elements, so we only check local path
 DIB_ELEMENTS_PATH=${DIB_REPO_PATH:-${OCTAVIA_REPO_PATH%/*}/diskimage-builder/elements}
 
-if ! [ -d $DIB_ELEMENTS_PATH ]; then
-    echo "ERROR: diskimage-builder elements directory not found at: " $DIB_ELEMENTS_PATH " Exiting."
-    exit 1
-fi
-
-ELEMENTS_REPO_PATH=${ELEMENTS_REPO_PATH:-${OCTAVIA_REPO_PATH%/*}/tripleo-image-elements}
-
-if ! [ -d $ELEMENTS_REPO_PATH ]; then
-    echo "ERROR: tripleo-image-elements repo directory not found at: " $ELEMENTS_REPO_PATH " Exiting."
-    exit 1
-fi
-
-TRIPLEO_ELEMENTS_PATH=${ELEMENTS_PATH:-$ELEMENTS_REPO_PATH/elements}
+TRIPLEO_ELEMENTS_PATH=${TRIPLEO_ELEMENTS_PATH:-${OCTAVIA_REPO_PATH%/*}/tripleo-image-elements/elements}
 
 if ! [ -d $TRIPLEO_ELEMENTS_PATH ]; then
-    echo "ERROR: tripleo-image-elements elements directory not found at: " $TRIPLEO_ELEMENTS_PATH " Exiting."
-    exit 1
+    SYSTEM_TRIPLEO_ELEMENTS_PATH=$(find_system_elements tripleo-image-elements)
+    if [ -z ${SYSTEM_TRIPLEO_ELEMENTS_PATH} ]; then
+        echo "ERROR: tripleo-image-elements elements directory not found at: " $TRIPLEO_ELEMENTS_PATH " Exiting."
+        exit 1
+    fi
+    TRIPLEO_ELEMENTS_PATH=${SYSTEM_TRIPLEO_ELEMENTS_PATH}
 fi
 
 if [ "$DIB_LOCAL_ELEMENTS_PATH" ]; then
@@ -201,6 +203,15 @@ fi
 
 export CLOUD_INIT_DATASOURCES=${CLOUD_INIT_DATASOURCES:-"ConfigDrive"}
 
+# Additional RHEL environment checks
+if [ "${AMP_BASEOS}" = "rhel" ]; then
+    if [ -z "${DIB_LOCAL_IMAGE}" ]; then
+        echo "DIB_LOCAL_IMAGE variable must be set and point to a RHEL 7 base cloud image. Exiting."
+        echo "For more information, see the README file in ${DIB_ELEMENTS_PATH}/elements/rhel7"
+        exit 1
+    fi
+fi
+
 # Find out what platform we are on
 if [ -e /etc/os-release ]; then
     platform=$(head -1 /etc/os-release)
@@ -213,7 +224,7 @@ else
 fi
 
 if [ "$AMP_ROOTPW" ] && [ "$platform" != 'NAME="Ubuntu"' ]; then
-    if [ "$(getenforce)" != "Disabled" ]; then
+    if [ "$(getenforce)" == "Enforcing" ]; then
         echo "A root password cannot be enabled for images built on this platform while SELinux is enabled."
         exit 1
     fi
@@ -245,7 +256,7 @@ if [ "$platform" = 'NAME="Ubuntu"' ]; then
 elif [ "$platform" = 'NAME=Fedora' ]; then
     PKG_LIST="qemu kpartx git"
     for pkg in $PKG_LIST; do
-        if ! yum list $pkg &> /dev/null; then
+        if ! yum list installed $pkg &> /dev/null; then
             echo "Required package " $pkg " is not installed.  Exiting."
             exit 1
         fi
@@ -254,7 +265,7 @@ else
     # centos or rhel
         PKG_LIST="qemu-kvm qemu-img kpartx git"
         for pkg in $PKG_LIST; do
-            if ! yum list $pkg &> /dev/null; then
+            if ! yum list installed $pkg &> /dev/null; then
                 echo "Required package " $pkg " is not installed.  Exiting."
                 exit 1
             fi
@@ -262,7 +273,7 @@ else
         if [ ${platform:0:6} = "CentOS" ]; then
             # install EPEL repo, in order to install argparse
             PKG_LIST="python-argparse"
-            if ! yum list $pkg &> /dev/null; then
+            if ! yum list installed $pkg &> /dev/null; then
                 echo "CentOS requires the python-argparse package be "
                 echo "installed separately from the EPEL repo."
                 echo "Required package " $pkg " is not installed.  Exiting."
@@ -271,6 +282,13 @@ else
         fi
 fi
 
+# pip may not be installed from package managers
+# only check that we find an executable
+if ! which pip &> /dev/null; then
+    echo "Required executable pip not found.  Exiting."
+    exit 1
+fi
+
 # "pip freeze" does not show argparse, even if it is explicitly installed,
 # because it is part of the standard python library in 2.7.
 # See https://github.com/pypa/pip/issues/1570
@@ -291,20 +309,6 @@ else
 fi
 pushd $TEMP > /dev/null
 
-# Setup some environment variables with the commit IDs used to build the image
-pushd $DIB_REPO_PATH > /dev/null
-export DIB_COMMIT_ID=`git rev-parse HEAD`
-popd > /dev/null
-
-pushd $ELEMENTS_REPO_PATH > /dev/null
-export ELEMENTS_COMMIT_ID=`git rev-parse HEAD`
-popd > /dev/null
-
-pushd $OCTAVIA_REPO_PATH > /dev/null
-export OCTAVIA_COMMIT_ID=`git rev-parse HEAD`
-popd > /dev/null
-
-
 # Setup the elements list
 
 if [ "$AMP_BASEOS" = "ubuntu" ]; then
@@ -328,6 +332,9 @@ elif [ "$AMP_BASEOS" = "centos" ]; then
         AMP_element_sequence="$AMP_element_sequence centos-mirror"
         export CENTOS_MIRROR="$BASE_OS_MIRROR"
     fi
+elif [ "$AMP_BASEOS" = "rhel" ]; then
+    AMP_element_sequence=${AMP_element_sequence:-"base vm rhel7"}
+    AMP_element_sequence="$AMP_element_sequence $AMP_BACKEND"
 fi
 
 if [ "$AMP_ROOTPW" ]; then
@@ -340,18 +347,19 @@ if [ "$AMP_BASEOS" = "ubuntu" ]; then
     AMP_element_sequence="$AMP_element_sequence rebind-sshd"
     AMP_element_sequence="$AMP_element_sequence no-resolvconf"
     AMP_element_sequence="$AMP_element_sequence amphora-agent-ubuntu"
-    AMP_element_sequence="$AMP_element_sequence pyroute2"
     AMP_element_sequence="$AMP_element_sequence keepalived-octavia-ubuntu"
 else
     AMP_element_sequence="$AMP_element_sequence no-resolvconf"
     AMP_element_sequence="$AMP_element_sequence amphora-agent"
-    AMP_element_sequence="$AMP_element_sequence pyroute2"
     AMP_element_sequence="$AMP_element_sequence keepalived-octavia"
 fi
 
 # Add pip-cache element
 AMP_element_sequence="$AMP_element_sequence pip-cache"
 
+# Add certificate ramfs ecrypt element
+AMP_element_sequence="$AMP_element_sequence cert-ramfs-ecrypt"
+
 # Allow full elements override
 if [ "$DIB_ELEMENTS" ]; then
     AMP_element_sequence="$DIB_ELEMENTS"
