@@ -30,7 +30,7 @@ from octavia import version
 LOG = logging.getLogger(__name__)
 
 core_opts = [
-    cfg.IPOpt('bind_host', default='0.0.0.0',
+    cfg.IPOpt('bind_host', default='127.0.0.1',
               help=_("The host IP to bind to")),
     cfg.PortOpt('bind_port', default=9876,
                 help=_("The port to bind to")),
@@ -50,8 +50,8 @@ core_opts = [
                 help=_("Allow the usage of the sorting")),
     cfg.StrOpt('pagination_max_limit', default="-1",
                help=_("The maximum number of items returned in a single "
-                      "response, value was 'infinite' or negative integer "
-                      "means no limit")),
+                      "response. The string 'infinite' or a negative "
+                      "integer value means 'no limit'")),
     cfg.StrOpt('host', default=utils.get_hostname(),
                help=_("The hostname Octavia is running on")),
     cfg.StrOpt('octavia_plugins',
@@ -86,11 +86,14 @@ networking_opts = [
                       'networking service.')),
     cfg.IntOpt('retry_interval', default=1,
                help=_('Seconds to wait before retrying an action with the '
-                      'networking service.'))
+                      'networking service.')),
+    cfg.IntOpt('port_detach_timeout', default=300,
+               help=_('Seconds to wait for a port to detach from an '
+                      'amphora.'))
 ]
 
 healthmanager_opts = [
-    cfg.IPOpt('bind_ip', default='0.0.0.0',
+    cfg.IPOpt('bind_ip', default='127.0.0.1',
               help=_('IP address the controller will listen on for '
                      'heart beats')),
     cfg.PortOpt('bind_port', default=5555,
@@ -123,7 +126,7 @@ healthmanager_opts = [
                 default=[]),
     cfg.IntOpt('heartbeat_interval',
                default=10,
-               help=_('Sleep time between sending hearthbeats.')),
+               help=_('Sleep time between sending heartbeats.')),
     cfg.StrOpt('event_streamer_driver',
                help=_('Specifies which driver to use for the event_streamer '
                       'for syncing the octavia and neutron_lbaas dbs. If you '
@@ -166,7 +169,7 @@ haproxy_amphora_opts = [
                       'suffixes.  Example: 10k')),
 
     # REST server
-    cfg.IPOpt('bind_host', default='0.0.0.0',
+    cfg.IPOpt('bind_host', default='0.0.0.0',  # nosec
               help=_("The host IP to bind to")),
     cfg.PortOpt('bind_port', default=9443,
                 help=_("The port to bind to")),
@@ -213,6 +216,10 @@ controller_worker_opts = [
                deprecated_for_removal=True,
                deprecated_reason='Superseded by amp_image_tag option.',
                help=_('Glance image id for the Amphora image to boot')),
+    cfg.StrOpt('amp_image_owner_id',
+               default='',
+               help=_('Restrict glance image selection to a specific '
+                      'owner ID.  This is a recommended security setting.')),
     cfg.StrOpt('amp_ssh_key_name',
                default='',
                help=_('SSH key name used to boot the Amphora')),
@@ -220,12 +227,19 @@ controller_worker_opts = [
                 default=True,
                 help=_('Determines whether or not to allow access '
                        'to the Amphorae')),
+    cfg.ListOpt('amp_boot_network_list',
+                default='',
+                help=_('List of networks to attach to the Amphorae. '
+                       'All networks defined in the list will '
+                       'be attached to each amphora.')),
     cfg.StrOpt('amp_network',
+               deprecated_for_removal=True,
+               deprecated_reason='Replaced by amp_boot_network_list.',
                default='',
-               help=_('Network to attach to the Amphora')),
+               help=_('Network to attach to the Amphorae.')),
     cfg.ListOpt('amp_secgroup_list',
                 default='',
-                help=_('List of security groups to attach to the Amphora')),
+                help=_('List of security groups to attach to the Amphora.')),
     cfg.StrOpt('client_ca',
                default='/etc/octavia/certs/ca_01.pem',
                help=_('Client CA for the amphora agent to use')),
@@ -296,6 +310,9 @@ house_keeping_opts = [
     cfg.IntOpt('amphora_expiry_age',
                default=604800,
                help=_('Amphora expiry age in seconds')),
+    cfg.IntOpt('load_balancer_expiry_age',
+               default=604800,
+               help=_('Load balancer expiry age in seconds')),
     cfg.IntOpt('cert_interval',
                default=3600,
                help=_('Certificate check interval in seconds')),
@@ -314,10 +331,8 @@ anchor_opts = [
                default='http://localhost:9999/v1/sign/default',
                help=_('Anchor URL')),
     cfg.StrOpt('username',
-               default='myusername',
                help=_('Anchor username')),
     cfg.StrOpt('password',
-               default='simplepassword',
                help=_('Anchor password'),
                secret=True)
 ]
@@ -332,11 +347,11 @@ keepalived_vrrp_opts = [
                help=_('VRRP health check script run interval in seconds.')),
     cfg.IntOpt('vrrp_fail_count',
                default=2,
-               help=_('Number of successive failure before transition to a '
+               help=_('Number of successive failures before transition to a '
                       'fail state.')),
     cfg.IntOpt('vrrp_success_count',
                default=2,
-               help=_('Number of successive failure before transition to a '
+               help=_('Number of consecutive successes before transition to a '
                       'success state.')),
     cfg.IntOpt('vrrp_garp_refresh_interval',
                default=5,
@@ -434,10 +449,8 @@ messaging.set_transport_defaults(control_exchange='octavia')
 _SQL_CONNECTION_DEFAULT = 'sqlite://'
 # Update the default QueuePool parameters. These can be tweaked by the
 # configuration variables - max_pool_size, max_overflow and pool_timeout
-db_options.set_defaults(cfg.CONF,
-                        connection=_SQL_CONNECTION_DEFAULT,
-                        sqlite_db='', max_pool_size=10,
-                        max_overflow=20, pool_timeout=10)
+db_options.set_defaults(cfg.CONF, connection=_SQL_CONNECTION_DEFAULT,
+                        max_pool_size=10, max_overflow=20, pool_timeout=10)
 
 logging.register_options(cfg.CONF)
 
