@@ -14,6 +14,7 @@
 
 import datetime
 
+from concurrent import futures
 from oslo_config import cfg
 from oslo_log import log as logging
 
@@ -41,8 +42,8 @@ class SpareAmphora(object):
         session = db_api.get_session()
         conf_spare_cnt = CONF.house_keeping.spare_amphora_pool_size
         curr_spare_cnt = self.amp_repo.get_spare_amphora_count(session)
-        LOG.debug("Required Spare Amphora count : %d" % conf_spare_cnt)
-        LOG.debug("Current Spare Amphora count : %d" % curr_spare_cnt)
+        LOG.debug("Required Spare Amphora count : %d", conf_spare_cnt)
+        LOG.debug("Current Spare Amphora count : %d", curr_spare_cnt)
         diff_count = conf_spare_cnt - curr_spare_cnt
 
         # When the current spare amphora is less than required
@@ -52,7 +53,7 @@ class SpareAmphora(object):
 
             # Call Amphora Create Flow diff_count times
             for i in range(1, diff_count + 1):
-                LOG.debug("Starting amphorae number %d ..." % i)
+                LOG.debug("Starting amphorae number %d ...", i)
                 self.cw.create_amphora()
 
         else:
@@ -64,6 +65,7 @@ class DatabaseCleanup(object):
     def __init__(self):
         self.amp_repo = repo.AmphoraRepository()
         self.amp_health_repo = repo.AmphoraHealthRepository()
+        self.lb_repo = repo.LoadBalancerRepository()
 
     def delete_old_amphorae(self):
         """Checks the DB for old amphora and deletes them based on it's age."""
@@ -76,6 +78,50 @@ class DatabaseCleanup(object):
         for amp in amphora:
             if self.amp_health_repo.check_amphora_expired(session, amp.id,
                                                           exp_age):
-                LOG.info(_LI('Attempting to delete Amphora id : %s') % amp.id)
+                LOG.info(_LI('Attempting to delete Amphora id : %s'), amp.id)
                 self.amp_repo.delete(session, id=amp.id)
                 LOG.info(_LI('Deleted Amphora id : %s') % amp.id)
+
+    def cleanup_load_balancers(self):
+        """Checks the DB for old load balancers and triggers their removal."""
+        exp_age = datetime.timedelta(
+            seconds=CONF.house_keeping.load_balancer_expiry_age)
+
+        session = db_api.get_session()
+        load_balancers = self.lb_repo.get_all(
+            session, provisioning_status=constants.DELETED)
+
+        for lb in load_balancers:
+            if self.lb_repo.check_load_balancer_expired(session, lb.id,
+                                                        exp_age):
+                LOG.info(_LI('Attempting to delete load balancer id : %s'),
+                         lb.id)
+                self.lb_repo.delete(session, id=lb.id)
+                LOG.info(_LI('Deleted load balancer id : %s') % lb.id)
+
+
+class CertRotation(object):
+    def __init__(self):
+        self.threads = CONF.house_keeping.cert_rotate_threads
+        self.cw = cw.ControllerWorker()
+
+    def rotate(self):
+        """Check the amphora db table for expiring auth certs."""
+        amp_repo = repo.AmphoraRepository()
+
+        with futures.ThreadPoolExecutor(max_workers=self.threads) as executor:
+            try:
+                session = db_api.get_session()
+                rotation_count = 0
+                while True:
+                    amp = amp_repo.get_cert_expiring_amphora(session)
+                    if not amp:
+                        break
+                    rotation_count += 1
+                    LOG.debug("Cert expired amphora's id is: %s", amp.id)
+                    executor.submit(self.cw.amphora_cert_rotation, amp.id)
+                if rotation_count > 0:
+                    LOG.info(_LI("Rotated certificates for %s amphora") %
+                             rotation_count)
+            finally:
+                executor.shutdown(wait=True)
