@@ -17,18 +17,23 @@ import logging
 
 from octavia.common import base_taskflow
 from octavia.common import constants
-from octavia.common import exceptions
 from octavia.controller.worker.flows import amphora_flows
 from octavia.controller.worker.flows import health_monitor_flows
+from octavia.controller.worker.flows import l7policy_flows
+from octavia.controller.worker.flows import l7rule_flows
 from octavia.controller.worker.flows import listener_flows
 from octavia.controller.worker.flows import load_balancer_flows
 from octavia.controller.worker.flows import member_flows
 from octavia.controller.worker.flows import pool_flows
 from octavia.db import api as db_apis
 from octavia.db import repositories as repo
+from octavia.i18n import _LE, _LI
 
+from oslo_config import cfg
+from oslo_utils import excutils
 from taskflow.listeners import logging as tf_logging
 
+CONF = cfg.CONF
 LOG = logging.getLogger(__name__)
 
 
@@ -42,6 +47,8 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         self._listener_flows = listener_flows.ListenerFlows()
         self._member_flows = member_flows.MemberFlows()
         self._pool_flows = pool_flows.PoolFlows()
+        self._l7policy_flows = l7policy_flows.L7PolicyFlows()
+        self._l7rule_flows = l7rule_flows.L7RuleFlows()
 
         self._amphora_repo = repo.AmphoraRepository()
         self._health_mon_repo = repo.HealthMonitorRepository()
@@ -49,18 +56,36 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         self._listener_repo = repo.ListenerRepository()
         self._member_repo = repo.MemberRepository()
         self._pool_repo = repo.PoolRepository()
+        self._l7policy_repo = repo.L7PolicyRepository()
+        self._l7rule_repo = repo.L7RuleRepository()
+
+        self._exclude_result_logging_tasks = (
+            constants.ROLE_STANDALONE + '-' +
+            constants.CREATE_AMP_FOR_LB_SUBFLOW + '-' +
+            constants.GENERATE_SERVER_PEM,
+            constants.ROLE_BACKUP + '-' +
+            constants.CREATE_AMP_FOR_LB_SUBFLOW + '-' +
+            constants.GENERATE_SERVER_PEM,
+            constants.ROLE_MASTER + '-' +
+            constants.CREATE_AMP_FOR_LB_SUBFLOW + '-' +
+            constants.GENERATE_SERVER_PEM,
+            constants.GENERATE_SERVER_PEM_TASK)
 
         super(ControllerWorker, self).__init__()
 
     def create_amphora(self):
         """Creates an Amphora.
 
+        This is used to create spare amphora.
+
         :returns: amphora_id
         """
         create_amp_tf = self._taskflow_load(self._amphora_flows.
                                             get_create_amphora_flow())
-        with tf_logging.DynamicLoggingListener(create_amp_tf,
-                                               log=LOG):
+        with tf_logging.DynamicLoggingListener(
+                create_amp_tf, log=LOG,
+                hide_inputs_outputs_of=self._exclude_result_logging_tasks):
+
             create_amp_tf.run()
 
         return create_amp_tf.storage.fetch('amphora')
@@ -76,7 +101,7 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
                                          id=amphora_id)
         delete_amp_tf = self._taskflow_load(self._amphora_flows.
                                             get_delete_amphora_flow(),
-                                            store={'amphora': amphora})
+                                            store={constants.AMPHORA: amphora})
         with tf_logging.DynamicLoggingListener(delete_amp_tf,
                                                log=LOG):
             delete_amp_tf.run()
@@ -91,19 +116,18 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         health_mon = self._health_mon_repo.get(db_apis.get_session(),
                                                pool_id=pool_id)
 
-        listener = health_mon.pool.listener
+        listeners = health_mon.pool.listeners
         health_mon.pool.health_monitor = health_mon
-        listener.default_pool = health_mon.pool
-        vip = health_mon.pool.listener.load_balancer.vip
-        load_balancer = health_mon.pool.listener.load_balancer
+        load_balancer = health_mon.pool.load_balancer
 
         create_hm_tf = self._taskflow_load(self._health_monitor_flows.
                                            get_create_health_monitor_flow(),
-                                           store={'health_mon': health_mon,
-                                                  'listener': listener,
-                                                  'loadbalancer':
-                                                      load_balancer,
-                                                  'vip': vip})
+                                           store={constants.HEALTH_MON:
+                                                  health_mon,
+                                                  constants.LISTENERS:
+                                                      listeners,
+                                                  constants.LOADBALANCER:
+                                                      load_balancer})
         with tf_logging.DynamicLoggingListener(create_hm_tf,
                                                log=LOG):
             create_hm_tf.run()
@@ -118,16 +142,14 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         health_mon = self._health_mon_repo.get(db_apis.get_session(),
                                                pool_id=pool_id)
 
-        listener = health_mon.pool.listener
-        listener.default_pool = health_mon.pool
-        vip = listener.load_balancer.vip
-        load_balancer = listener.load_balancer
+        listeners = health_mon.pool.listeners
+        load_balancer = health_mon.pool.load_balancer
 
         delete_hm_tf = self._taskflow_load(
             self._health_monitor_flows.get_delete_health_monitor_flow(),
-            store={'health_mon': health_mon, 'pool_id': pool_id,
-                   'listener': listener, 'loadbalancer': load_balancer,
-                   'vip': vip})
+            store={constants.HEALTH_MON: health_mon, constants.POOL_ID:
+                   pool_id, constants.LISTENERS: listeners,
+                   constants.LOADBALANCER: load_balancer})
         with tf_logging.DynamicLoggingListener(delete_hm_tf,
                                                log=LOG):
             delete_hm_tf.run()
@@ -143,20 +165,19 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         health_mon = self._health_mon_repo.get(db_apis.get_session(),
                                                pool_id=pool_id)
 
-        listener = health_mon.pool.listener
+        listeners = health_mon.pool.listeners
         health_mon.pool.health_monitor = health_mon
-        listener.default_pool = health_mon.pool
-        vip = health_mon.pool.listener.load_balancer.vip
-        load_balancer = health_mon.pool.listener.load_balancer
+        load_balancer = health_mon.pool.load_balancer
 
         update_hm_tf = self._taskflow_load(self._health_monitor_flows.
                                            get_update_health_monitor_flow(),
-                                           store={'health_mon': health_mon,
-                                                  'listener': listener,
-                                                  'loadbalancer':
+                                           store={constants.HEALTH_MON:
+                                                  health_mon,
+                                                  constants.LISTENERS:
+                                                      listeners,
+                                                  constants.LOADBALANCER:
                                                       load_balancer,
-                                                  'vip': vip,
-                                                  'update_dict':
+                                                  constants.UPDATE_DICT:
                                                       health_monitor_updates})
         with tf_logging.DynamicLoggingListener(update_hm_tf,
                                                log=LOG):
@@ -172,14 +193,13 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         listener = self._listener_repo.get(db_apis.get_session(),
                                            id=listener_id)
         load_balancer = listener.load_balancer
-        vip = listener.load_balancer.vip
 
         create_listener_tf = self._taskflow_load(self._listener_flows.
                                                  get_create_listener_flow(),
-                                                 store={'listener': listener,
-                                                        'loadbalancer':
-                                                            load_balancer,
-                                                        'vip': vip})
+                                                 store={constants.LOADBALANCER:
+                                                        load_balancer,
+                                                        constants.LISTENERS:
+                                                            [listener]})
         with tf_logging.DynamicLoggingListener(create_listener_tf,
                                                log=LOG):
             create_listener_tf.run()
@@ -194,12 +214,11 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         listener = self._listener_repo.get(db_apis.get_session(),
                                            id=listener_id)
         load_balancer = listener.load_balancer
-        vip = load_balancer.vip
 
         delete_listener_tf = self._taskflow_load(
             self._listener_flows.get_delete_listener_flow(),
             store={constants.LOADBALANCER: load_balancer,
-                   constants.LISTENER: listener, constants.VIP: vip})
+                   constants.LISTENER: listener})
         with tf_logging.DynamicLoggingListener(delete_listener_tf,
                                                log=LOG):
             delete_listener_tf.run()
@@ -216,16 +235,17 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
                                            id=listener_id)
 
         load_balancer = listener.load_balancer
-        vip = listener.load_balancer.vip
 
         update_listener_tf = self._taskflow_load(self._listener_flows.
                                                  get_update_listener_flow(),
-                                                 store={'listener': listener,
-                                                        'vip': vip,
-                                                        'loadbalancer':
+                                                 store={constants.LISTENER:
+                                                        listener,
+                                                        constants.LOADBALANCER:
                                                             load_balancer,
-                                                        'update_dict':
-                                                            listener_updates})
+                                                        constants.UPDATE_DICT:
+                                                            listener_updates,
+                                                        constants.LISTENERS:
+                                                            [listener]})
         with tf_logging.DynamicLoggingListener(update_listener_tf, log=LOG):
             update_listener_tf.run()
 
@@ -233,7 +253,7 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         """Creates a load balancer by allocating Amphorae.
 
         First tries to allocate an existing Amphora in READY state.
-        If none are available it will attempt to build one specificly
+        If none are available it will attempt to build one specifically
         for this load balancer.
 
         :param load_balancer_id: ID of the load balancer to create
@@ -241,31 +261,25 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         :raises NoSuitableAmphoraException: Unable to allocate an Amphora.
         """
 
-        # Note this is a bit strange in how it handles building
-        # Amphora if there are no spares.  TaskFlow has a spec for
-        # a conditional flow that would make this cleaner once implemented.
-        # https://review.openstack.org/#/c/98946/
-
         store = {constants.LOADBALANCER_ID: load_balancer_id}
-        create_lb_tf = self._taskflow_load(
-            self._lb_flows.get_create_load_balancer_flow(), store=store)
-        with tf_logging.DynamicLoggingListener(create_lb_tf,
-                                               log=LOG):
-            try:
-                create_lb_tf.run()
-            except exceptions.NoReadyAmphoraeException:
-                create_amp_lb_tf = self._taskflow_load(
-                    self._amphora_flows.get_create_amphora_for_lb_flow(),
-                    store=store)
 
-                with tf_logging.DynamicLoggingListener(create_amp_lb_tf,
-                                                       log=LOG):
-                    try:
-                        create_amp_lb_tf.run()
-                    except exceptions.ComputeBuildException as e:
-                        raise exceptions.NoSuitableAmphoraException(msg=e.msg)
+        topology = CONF.controller_worker.loadbalancer_topology
 
-    def delete_load_balancer(self, load_balancer_id):
+        store[constants.UPDATE_DICT] = {
+            constants.LOADBALANCER_TOPOLOGY: topology
+        }
+
+        lb = self._lb_repo.get(db_apis.get_session(), id=load_balancer_id)
+        create_lb_flow = self._lb_flows.get_create_load_balancer_flow(
+            topology=topology, listeners=lb.listeners)
+
+        create_lb_tf = self._taskflow_load(create_lb_flow, store=store)
+        with tf_logging.DynamicLoggingListener(
+                create_lb_tf, log=LOG,
+                hide_inputs_outputs_of=self._exclude_result_logging_tasks):
+            create_lb_tf.run()
+
+    def delete_load_balancer(self, load_balancer_id, cascade=False):
         """Deletes a load balancer by de-allocating Amphorae.
 
         :param load_balancer_id: ID of the load balancer to delete
@@ -275,9 +289,14 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         lb = self._lb_repo.get(db_apis.get_session(),
                                id=load_balancer_id)
 
-        delete_lb_tf = self._taskflow_load(self._lb_flows.
-                                           get_delete_load_balancer_flow(),
-                                           store={'loadbalancer': lb})
+        if cascade:
+            (flow,
+             store) = self._lb_flows.get_cascade_delete_load_balancer_flow(lb)
+        else:
+            (flow, store) = self._lb_flows.get_delete_load_balancer_flow(lb)
+        store.update({constants.LOADBALANCER: lb,
+                      constants.SERVER_GROUP_ID: lb.server_group_id})
+        delete_lb_tf = self._taskflow_load(flow, store=store)
 
         with tf_logging.DynamicLoggingListener(delete_lb_tf,
                                                log=LOG):
@@ -293,10 +312,15 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         """
         lb = self._lb_repo.get(db_apis.get_session(),
                                id=load_balancer_id)
+        listeners = self._listener_repo.get_all(
+            db_apis.get_session(),
+            load_balancer_id=load_balancer_id)
 
         update_lb_tf = self._taskflow_load(
             self._lb_flows.get_update_load_balancer_flow(),
-            store={'loadbalancer': lb, 'update_dict': load_balancer_updates})
+            store={constants.LOADBALANCER: lb,
+                   constants.LISTENERS: listeners,
+                   constants.UPDATE_DICT: load_balancer_updates})
 
         with tf_logging.DynamicLoggingListener(update_lb_tf,
                                                log=LOG):
@@ -312,18 +336,16 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         member = self._member_repo.get(db_apis.get_session(),
                                        id=member_id)
 
-        listener = member.pool.listener
-        listener.default_pool = member.pool
-        load_balancer = listener.load_balancer
-        vip = listener.load_balancer.vip
+        listeners = member.pool.listeners
+        load_balancer = member.pool.load_balancer
 
         create_member_tf = self._taskflow_load(self._member_flows.
                                                get_create_member_flow(),
-                                               store={'member': member,
-                                                      'listener': listener,
-                                                      'loadbalancer':
-                                                          load_balancer,
-                                                      'vip': vip})
+                                               store={constants.MEMBER: member,
+                                                      constants.LISTENERS:
+                                                          listeners,
+                                                      constants.LOADBALANCER:
+                                                          load_balancer})
         with tf_logging.DynamicLoggingListener(create_member_tf,
                                                log=LOG):
             create_member_tf.run()
@@ -338,16 +360,13 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         member = self._member_repo.get(db_apis.get_session(),
                                        id=member_id)
 
-        listener = member.pool.listener
-        listener.default_pool = member.pool
-        load_balancer = listener.load_balancer
-        vip = load_balancer.vip
+        listeners = member.pool.listeners
+        load_balancer = member.pool.load_balancer
 
         delete_member_tf = self._taskflow_load(
             self._member_flows.get_delete_member_flow(),
-            store={'member': member, 'member_id': member_id,
-                   'listener': listener, 'vip': vip,
-                   'loadbalancer': load_balancer})
+            store={constants.MEMBER: member, constants.LISTENERS: listeners,
+                   constants.LOADBALANCER: load_balancer})
         with tf_logging.DynamicLoggingListener(delete_member_tf,
                                                log=LOG):
             delete_member_tf.run()
@@ -363,19 +382,17 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         member = self._member_repo.get(db_apis.get_session(),
                                        id=member_id)
 
-        listener = member.pool.listener
-        listener.default_pool = member.pool
-        load_balancer = listener.load_balancer
-        vip = listener.load_balancer.vip
+        listeners = member.pool.listeners
+        load_balancer = member.pool.load_balancer
 
         update_member_tf = self._taskflow_load(self._member_flows.
                                                get_update_member_flow(),
-                                               store={'member': member,
-                                                      'listener': listener,
-                                                      'loadbalancer':
+                                               store={constants.MEMBER: member,
+                                                      constants.LISTENERS:
+                                                          listeners,
+                                                      constants.LOADBALANCER:
                                                           load_balancer,
-                                                      'vip': vip,
-                                                      'update_dict':
+                                                      constants.UPDATE_DICT:
                                                           member_updates})
         with tf_logging.DynamicLoggingListener(update_member_tf,
                                                log=LOG):
@@ -391,18 +408,16 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         pool = self._pool_repo.get(db_apis.get_session(),
                                    id=pool_id)
 
-        listener = pool.listener
-        listener.default_pool = pool
-        load_balancer = listener.load_balancer
-        vip = listener.load_balancer.vip
+        listeners = pool.listeners
+        load_balancer = pool.load_balancer
 
         create_pool_tf = self._taskflow_load(self._pool_flows.
                                              get_create_pool_flow(),
-                                             store={'pool': pool,
-                                                    'listener': listener,
-                                                    'loadbalancer':
-                                                        load_balancer,
-                                                    'vip': vip})
+                                             store={constants.POOL: pool,
+                                                    constants.LISTENERS:
+                                                        listeners,
+                                                    constants.LOADBALANCER:
+                                                        load_balancer})
         with tf_logging.DynamicLoggingListener(create_pool_tf,
                                                log=LOG):
             create_pool_tf.run()
@@ -417,15 +432,13 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         pool = self._pool_repo.get(db_apis.get_session(),
                                    id=pool_id)
 
-        load_balancer = pool.listener.load_balancer
-        listener = pool.listener
-        vip = load_balancer.vip
+        load_balancer = pool.load_balancer
+        listeners = pool.listeners
 
         delete_pool_tf = self._taskflow_load(
             self._pool_flows.get_delete_pool_flow(),
-            store={constants.POOL: pool, constants.LISTENER: listener,
-                   constants.LOADBALANCER: load_balancer,
-                   constants.VIP: vip})
+            store={constants.POOL: pool, constants.LISTENERS: listeners,
+                   constants.LOADBALANCER: load_balancer})
         with tf_logging.DynamicLoggingListener(delete_pool_tf,
                                                log=LOG):
             delete_pool_tf.run()
@@ -441,24 +454,158 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         pool = self._pool_repo.get(db_apis.get_session(),
                                    id=pool_id)
 
-        listener = pool.listener
-        listener.default_pool = pool
-        load_balancer = listener.load_balancer
-        vip = listener.load_balancer.vip
+        listeners = pool.listeners
+        load_balancer = pool.load_balancer
 
         update_pool_tf = self._taskflow_load(self._pool_flows.
                                              get_update_pool_flow(),
-                                             store={'pool': pool,
-                                                    'listener': listener,
-                                                    'loadbalancer':
+                                             store={constants.POOL: pool,
+                                                    constants.LISTENERS:
+                                                        listeners,
+                                                    constants.LOADBALANCER:
                                                         load_balancer,
-                                                    'vip': vip,
-                                                    'update_dict':
+                                                    constants.UPDATE_DICT:
                                                         pool_updates})
         with tf_logging.DynamicLoggingListener(update_pool_tf,
                                                log=LOG):
             update_pool_tf.run()
 
+    def create_l7policy(self, l7policy_id):
+        """Creates an L7 Policy.
+
+        :param l7policy_id: ID of the l7policy to create
+        :returns: None
+        :raises NoSuitableLB: Unable to find the load balancer
+        """
+        l7policy = self._l7policy_repo.get(db_apis.get_session(),
+                                           id=l7policy_id)
+
+        listeners = [l7policy.listener]
+        load_balancer = l7policy.listener.load_balancer
+
+        create_l7policy_tf = self._taskflow_load(
+            self._l7policy_flows.get_create_l7policy_flow(),
+            store={constants.L7POLICY: l7policy,
+                   constants.LISTENERS: listeners,
+                   constants.LOADBALANCER: load_balancer})
+        with tf_logging.DynamicLoggingListener(create_l7policy_tf,
+                                               log=LOG):
+            create_l7policy_tf.run()
+
+    def delete_l7policy(self, l7policy_id):
+        """Deletes an L7 policy.
+
+        :param l7policy_id: ID of the l7policy to delete
+        :returns: None
+        :raises L7PolicyNotFound: The referenced l7policy was not found
+        """
+        l7policy = self._l7policy_repo.get(db_apis.get_session(),
+                                           id=l7policy_id)
+
+        load_balancer = l7policy.listener.load_balancer
+        listeners = [l7policy.listener]
+
+        delete_l7policy_tf = self._taskflow_load(
+            self._l7policy_flows.get_delete_l7policy_flow(),
+            store={constants.L7POLICY: l7policy,
+                   constants.LISTENERS: listeners,
+                   constants.LOADBALANCER: load_balancer})
+        with tf_logging.DynamicLoggingListener(delete_l7policy_tf,
+                                               log=LOG):
+            delete_l7policy_tf.run()
+
+    def update_l7policy(self, l7policy_id, l7policy_updates):
+        """Updates an L7 policy.
+
+        :param l7policy_id: ID of the l7policy to update
+        :param l7policy_updates: Dict containing updated l7policy attributes
+        :returns: None
+        :raises L7PolicyNotFound: The referenced l7policy was not found
+        """
+        l7policy = self._l7policy_repo.get(db_apis.get_session(),
+                                           id=l7policy_id)
+
+        listeners = [l7policy.listener]
+        load_balancer = l7policy.listener.load_balancer
+
+        update_l7policy_tf = self._taskflow_load(
+            self._l7policy_flows.get_update_l7policy_flow(),
+            store={constants.L7POLICY: l7policy,
+                   constants.LISTENERS: listeners,
+                   constants.LOADBALANCER: load_balancer,
+                   constants.UPDATE_DICT: l7policy_updates})
+        with tf_logging.DynamicLoggingListener(update_l7policy_tf,
+                                               log=LOG):
+            update_l7policy_tf.run()
+
+    def create_l7rule(self, l7rule_id):
+        """Creates an L7 Rule.
+
+        :param l7rule_id: ID of the l7rule to create
+        :returns: None
+        :raises NoSuitableLB: Unable to find the load balancer
+        """
+        l7rule = self._l7rule_repo.get(db_apis.get_session(),
+                                       id=l7rule_id)
+
+        listeners = [l7rule.l7policy.listener]
+        load_balancer = l7rule.l7policy.listener.load_balancer
+
+        create_l7rule_tf = self._taskflow_load(
+            self._l7rule_flows.get_create_l7rule_flow(),
+            store={constants.L7RULE: l7rule,
+                   constants.LISTENERS: listeners,
+                   constants.LOADBALANCER: load_balancer})
+        with tf_logging.DynamicLoggingListener(create_l7rule_tf,
+                                               log=LOG):
+            create_l7rule_tf.run()
+
+    def delete_l7rule(self, l7rule_id):
+        """Deletes an L7 rule.
+
+        :param l7rule_id: ID of the l7rule to delete
+        :returns: None
+        :raises L7RuleNotFound: The referenced l7rule was not found
+        """
+        l7rule = self._l7rule_repo.get(db_apis.get_session(),
+                                       id=l7rule_id)
+
+        load_balancer = l7rule.l7policy.listener.load_balancer
+        listeners = [l7rule.l7policy.listener]
+
+        delete_l7rule_tf = self._taskflow_load(
+            self._l7rule_flows.get_delete_l7rule_flow(),
+            store={constants.L7RULE: l7rule,
+                   constants.LISTENERS: listeners,
+                   constants.LOADBALANCER: load_balancer})
+        with tf_logging.DynamicLoggingListener(delete_l7rule_tf,
+                                               log=LOG):
+            delete_l7rule_tf.run()
+
+    def update_l7rule(self, l7rule_id, l7rule_updates):
+        """Updates an L7 rule.
+
+        :param l7rule_id: ID of the l7rule to update
+        :param l7rule_updates: Dict containing updated l7rule attributes
+        :returns: None
+        :raises L7RuleNotFound: The referenced l7rule was not found
+        """
+        l7rule = self._l7rule_repo.get(db_apis.get_session(),
+                                       id=l7rule_id)
+
+        listeners = [l7rule.l7policy.listener]
+        load_balancer = l7rule.l7policy.listener.load_balancer
+
+        update_l7rule_tf = self._taskflow_load(
+            self._l7rule_flows.get_update_l7rule_flow(),
+            store={constants.L7RULE: l7rule,
+                   constants.LISTENERS: listeners,
+                   constants.LOADBALANCER: load_balancer,
+                   constants.UPDATE_DICT: l7rule_updates})
+        with tf_logging.DynamicLoggingListener(update_l7rule_tf,
+                                               log=LOG):
+            update_l7rule_tf.run()
+
     def failover_amphora(self, amphora_id):
         """Perform failover operations for an amphora.
 
@@ -467,13 +614,42 @@ class ControllerWorker(base_taskflow.BaseTaskFlowEngine):
         :raises AmphoraNotFound: The referenced amphora was not found
         """
 
+        try:
+            amp = self._amphora_repo.get(db_apis.get_session(),
+                                         id=amphora_id)
+
+            failover_amphora_tf = self._taskflow_load(
+                self._amphora_flows.get_failover_flow(role=amp.role),
+                store={constants.FAILED_AMPHORA: amp,
+                       constants.LOADBALANCER_ID: amp.load_balancer_id})
+            with tf_logging.DynamicLoggingListener(
+                    failover_amphora_tf, log=LOG,
+                    hide_inputs_outputs_of=self._exclude_result_logging_tasks):
+
+                failover_amphora_tf.run()
+
+        except Exception as e:
+            with excutils.save_and_reraise_exception():
+                LOG.error(_LE("Failover exception: %s") % e)
+
+    def amphora_cert_rotation(self, amphora_id):
+        """Perform cert rotation for an amphora.
+
+        :param amphora_id: ID for amphora to rotate
+        :returns: None
+        :raises AmphoraNotFound: The referenced amphora was not found
+        """
+
         amp = self._amphora_repo.get(db_apis.get_session(),
                                      id=amphora_id)
+        LOG.info(_LI("Start amphora cert rotation, amphora's id is: %s")
+                 % amp.id)
 
-        failover_amphora_tf = self._taskflow_load(
-            self._amphora_flows.get_failover_flow(),
+        certrotation_amphora_tf = self._taskflow_load(
+            self._amphora_flows.cert_rotate_amphora_flow(),
             store={constants.AMPHORA: amp,
-                   constants.LOADBALANCER_ID: amp.load_balancer_id})
-        with tf_logging.DynamicLoggingListener(failover_amphora_tf,
+                   constants.AMPHORA_ID: amp.id})
+
+        with tf_logging.DynamicLoggingListener(certrotation_amphora_tf,
                                                log=LOG):
-            failover_amphora_tf.run()
+            certrotation_amphora_tf.run()
