@@ -22,6 +22,7 @@ from octavia.controller.worker.tasks import amphora_driver_tasks
 from octavia.controller.worker.tasks import cert_task
 from octavia.controller.worker.tasks import compute_tasks
 from octavia.controller.worker.tasks import database_tasks
+from octavia.controller.worker.tasks import lifecycle_tasks
 from octavia.controller.worker.tasks import network_tasks
 
 CONF = cfg.CONF
@@ -38,15 +39,13 @@ class AmphoraFlows(object):
     def get_create_amphora_flow(self):
         """Creates a flow to create an amphora.
 
-        Ideally that should be configurable in the
-        config file - a db session needs to be placed
-        into the flow
-
         :returns: The flow for creating the amphora
         """
         create_amphora_flow = linear_flow.Flow(constants.CREATE_AMPHORA_FLOW)
         create_amphora_flow.add(database_tasks.CreateAmphoraInDB(
                                 provides=constants.AMPHORA_ID))
+        create_amphora_flow.add(lifecycle_tasks.AmphoraIDToErrorOnRevertTask(
+            requires=constants.AMPHORA_ID))
         if self.REST_AMPHORA_DRIVER:
             create_amphora_flow.add(cert_task.GenerateServerPEMTask(
                                     provides=constants.SERVER_PEM))
@@ -202,15 +201,15 @@ class AmphoraFlows(object):
         :return: True if a spare amphora exists in DB
         """
 
-        return history.values()[0] is not None
+        return list(history.values())[0] is not None
 
     def _create_new_amp_for_lb_decider(self, history):
         """decides if a new amphora must be created for the lb
 
-        :return: True
+        :return: True if there is no spare amphora
         """
 
-        return history.values()[0] is None
+        return list(history.values())[0] is None
 
     def get_amphora_for_lb_subflow(
             self, prefix, role=constants.ROLE_STANDALONE):
@@ -230,7 +229,7 @@ class AmphoraFlows(object):
             requires=constants.LOADBALANCER_ID,
             provides=constants.AMPHORA_ID)
 
-        # Define a subflow for if we successfuly map an amphora
+        # Define a subflow for if we successfully map an amphora
         map_lb_to_amp = self._get_post_map_lb_subflow(prefix, role)
         # Define a subflow for if we can't map an amphora
         create_amp = self._get_create_amp_for_lb_subflow(prefix, role)
@@ -259,6 +258,8 @@ class AmphoraFlows(object):
         """
 
         delete_amphora_flow = linear_flow.Flow(constants.DELETE_AMPHORA_FLOW)
+        delete_amphora_flow.add(lifecycle_tasks.AmphoraToErrorOnRevertTask(
+            requires=constants.AMPHORA))
         delete_amphora_flow.add(database_tasks.
                                 MarkAmphoraPendingDeleteInDB(
                                     requires=constants.AMPHORA))
@@ -283,11 +284,8 @@ class AmphoraFlows(object):
 
         failover_amphora_flow = linear_flow.Flow(
             constants.FAILOVER_AMPHORA_FLOW)
-        failover_amphora_flow.add(
-            network_tasks.RetrievePortIDsOnAmphoraExceptLBNetwork(
-                rebind={constants.AMPHORA: constants.FAILED_AMPHORA},
-                requires=constants.AMPHORA, provides=constants.PORTS))
-        failover_amphora_flow.add(network_tasks.FailoverPreparationForAmphora(
+
+        failover_amphora_flow.add(lifecycle_tasks.AmphoraToErrorOnRevertTask(
             rebind={constants.AMPHORA: constants.FAILED_AMPHORA},
             requires=constants.AMPHORA))
 
@@ -303,6 +301,9 @@ class AmphoraFlows(object):
         failover_amphora_flow.add(compute_tasks.ComputeDelete(
             rebind={constants.AMPHORA: constants.FAILED_AMPHORA},
             requires=constants.AMPHORA))
+        failover_amphora_flow.add(network_tasks.WaitForPortDetach(
+            rebind={constants.AMPHORA: constants.FAILED_AMPHORA},
+            requires=constants.AMPHORA))
         failover_amphora_flow.add(
             database_tasks.DisableAmphoraHealthMonitoring(
                 rebind={constants.AMPHORA: constants.FAILED_AMPHORA},
@@ -337,23 +338,25 @@ class AmphoraFlows(object):
             provides=constants.AMPHORAE_NETWORK_CONFIG))
         failover_amphora_flow.add(database_tasks.GetListenersFromLoadbalancer(
             requires=constants.LOADBALANCER, provides=constants.LISTENERS))
-        failover_amphora_flow.add(database_tasks.GetVipFromLoadbalancer(
-            requires=constants.LOADBALANCER, provides=constants.VIP))
+
         failover_amphora_flow.add(amphora_driver_tasks.ListenersUpdate(
             requires=(constants.LOADBALANCER, constants.LISTENERS)))
-        failover_amphora_flow.add(network_tasks.PlugPorts(
-            requires=(constants.AMPHORA, constants.PORTS)))
+
+        # Plug the VIP ports into the new amphora
+        failover_amphora_flow.add(network_tasks.PlugVIPPort(
+            requires=(constants.AMPHORA, constants.AMPHORAE_NETWORK_CONFIG)))
         failover_amphora_flow.add(amphora_driver_tasks.AmphoraPostVIPPlug(
-            requires=(constants.LOADBALANCER,
+            requires=(constants.AMPHORA, constants.LOADBALANCER,
                       constants.AMPHORAE_NETWORK_CONFIG)))
-        failover_amphora_flow.add(
-            network_tasks.GetMemberPorts(
-                requires=(constants.LOADBALANCER, constants.AMPHORA),
-                provides=constants.MEMBER_PORTS
-            ))
-        failover_amphora_flow.add(amphora_driver_tasks.AmphoraPostNetworkPlug(
-            rebind={constants.PORTS: constants.MEMBER_PORTS},
-            requires=(constants.AMPHORA, constants.PORTS)))
+
+        # Plug the member networks into the new amphora
+        failover_amphora_flow.add(network_tasks.CalculateDelta(
+            requires=(constants.LOADBALANCER),
+            provides=constants.DELTAS))
+        failover_amphora_flow.add(network_tasks.HandleNetworkDeltas(
+            requires=constants.DELTAS, provides=constants.ADDED_PORTS))
+        failover_amphora_flow.add(amphora_driver_tasks.AmphoraePostNetworkPlug(
+            requires=(constants.LOADBALANCER, constants.ADDED_PORTS)))
 
         # Handle the amphora role and VRRP if necessary
         if role == constants.ROLE_MASTER:
@@ -411,6 +414,9 @@ class AmphoraFlows(object):
         rotated_amphora_flow = linear_flow.Flow(
             constants.CERT_ROTATE_AMPHORA_FLOW)
 
+        rotated_amphora_flow.add(lifecycle_tasks.AmphoraToErrorOnRevertTask(
+            requires=constants.AMPHORA))
+
         # create a new certificate, the returned value is the newly created
         # certificate
         rotated_amphora_flow.add(cert_task.GenerateServerPEMTask(
