@@ -16,14 +16,16 @@
 import logging
 
 from oslo_config import cfg
+import six
 from stevedore import driver as stevedore_driver
 from taskflow import task
 from taskflow.types import failure
 
 from octavia.common import constants
+from octavia.controller.worker import task_utils as task_utilities
 from octavia.db import api as db_apis
 from octavia.db import repositories as repo
-from octavia.i18n import _LW
+from octavia.i18n import _LE, _LW
 
 CONF = cfg.CONF
 CONF.import_group('controller_worker', 'octavia.common.config')
@@ -43,110 +45,98 @@ class BaseAmphoraTask(task.Task):
         self.amphora_repo = repo.AmphoraRepository()
         self.listener_repo = repo.ListenerRepository()
         self.loadbalancer_repo = repo.LoadBalancerRepository()
-
-
-class ListenerUpdate(BaseAmphoraTask):
-    """Task to update an amphora with new configuration for the listener."""
-
-    def execute(self, listener, vip):
-        """Execute listener update routines for an amphora."""
-        self.amphora_driver.update(listener, vip)
-        LOG.debug("Updated amphora with new configuration for listener")
-
-    def revert(self, listener, *args, **kwargs):
-        """Handle a failed listener update."""
-
-        LOG.warn(_LW("Reverting listener update."))
-        self.listener_repo.update(db_apis.get_session(), id=listener.id,
-                                  provisioning_status=constants.ERROR)
-        return None
+        self.task_utils = task_utilities.TaskUtils()
 
 
 class ListenersUpdate(BaseAmphoraTask):
-    """Task to update amphora with all listeners' configurations."""
+    """Task to update amphora with all specified listeners' configurations."""
 
-    def execute(self, listeners, vip):
+    def execute(self, loadbalancer, listeners):
         """Execute updates per listener for an amphora."""
         for listener in listeners:
-            self.amphora_driver.update(listener, vip)
+            listener.load_balancer = loadbalancer
+            self.amphora_driver.update(listener, loadbalancer.vip)
 
-    def revert(self, listeners, *args, **kwargs):
+    def revert(self, loadbalancer, *args, **kwargs):
         """Handle failed listeners updates."""
 
-        LOG.warn(_LW("Reverting listeners updates."))
-        for listener in listeners:
-            self.listener_repo.update(db_apis.get_session(), id=listener.id,
-                                      provisioning_status=constants.ERROR)
+        LOG.warning(_LW("Reverting listeners updates."))
+
+        for listener in loadbalancer.listeners:
+            self.task_utils.mark_listener_prov_status_error(listener.id)
+
         return None
 
 
 class ListenerStop(BaseAmphoraTask):
     """Task to stop the listener on the vip."""
 
-    def execute(self, listener, vip):
+    def execute(self, loadbalancer, listener):
         """Execute listener stop routines for an amphora."""
-        self.amphora_driver.stop(listener, vip)
+        self.amphora_driver.stop(listener, loadbalancer.vip)
         LOG.debug("Stopped the listener on the vip")
 
     def revert(self, listener, *args, **kwargs):
         """Handle a failed listener stop."""
 
-        LOG.warn(_LW("Reverting listener stop."))
-        self.listener_repo.update(db_apis.get_session(), id=listener.id,
-                                  provisioning_status=constants.ERROR)
+        LOG.warning(_LW("Reverting listener stop."))
+
+        self.task_utils.mark_listener_prov_status_error(listener.id)
+
         return None
 
 
 class ListenerStart(BaseAmphoraTask):
     """Task to start the listener on the vip."""
 
-    def execute(self, listener, vip):
+    def execute(self, loadbalancer, listener):
         """Execute listener start routines for an amphora."""
-        self.amphora_driver.start(listener, vip)
+        self.amphora_driver.start(listener, loadbalancer.vip)
         LOG.debug("Started the listener on the vip")
 
     def revert(self, listener, *args, **kwargs):
         """Handle a failed listener start."""
 
-        LOG.warn(_LW("Reverting listener start."))
-        self.listener_repo.update(db_apis.get_session(), id=listener.id,
-                                  provisioning_status=constants.ERROR)
+        LOG.warning(_LW("Reverting listener start."))
+
+        self.task_utils.mark_listener_prov_status_error(listener.id)
+
         return None
 
 
 class ListenersStart(BaseAmphoraTask):
     """Task to start all listeners on the vip."""
 
-    def execute(self, listeners, vip):
+    def execute(self, loadbalancer, listeners):
         """Execute listener start routines for listeners on an amphora."""
         for listener in listeners:
-            self.amphora_driver.start(listener, vip)
+            self.amphora_driver.start(listener, loadbalancer.vip)
         LOG.debug("Started the listeners on the vip")
 
     def revert(self, listeners, *args, **kwargs):
         """Handle failed listeners starts."""
 
-        LOG.warn(_LW("Reverting listeners starts."))
+        LOG.warning(_LW("Reverting listeners starts."))
         for listener in listeners:
-            self.listener_repo.update(db_apis.get_session(), id=listener.id,
-                                      provisioning_status=constants.ERROR)
+            self.task_utils.mark_listener_prov_status_error(listener.id)
+
         return None
 
 
 class ListenerDelete(BaseAmphoraTask):
     """Task to delete the listener on the vip."""
 
-    def execute(self, listener, vip):
+    def execute(self, loadbalancer, listener):
         """Execute listener delete routines for an amphora."""
-        self.amphora_driver.delete(listener, vip)
+        self.amphora_driver.delete(listener, loadbalancer.vip)
         LOG.debug("Deleted the listener on the vip")
 
     def revert(self, listener, *args, **kwargs):
         """Handle a failed listener delete."""
 
-        LOG.warn(_LW("Reverting listener delete."))
-        self.listener_repo.update(db_apis.get_session(), id=listener.id,
-                                  provisioning_status=constants.ERROR)
+        LOG.warning(_LW("Reverting listener delete."))
+
+        self.task_utils.mark_listener_prov_status_error(listener.id)
 
 
 class AmphoraGetInfo(BaseAmphoraTask):
@@ -177,9 +167,8 @@ class AmphoraFinalize(BaseAmphoraTask):
         """Handle a failed amphora finalize."""
         if isinstance(result, failure.Failure):
             return
-        LOG.warn(_LW("Reverting amphora finalize."))
-        self.amphora_repo.update(db_apis.get_session(), id=amphora.id,
-                                 status=constants.ERROR)
+        LOG.warning(_LW("Reverting amphora finalize."))
+        self.task_utils.mark_amphora_status_error(amphora.id)
 
 
 class AmphoraPostNetworkPlug(BaseAmphoraTask):
@@ -190,16 +179,15 @@ class AmphoraPostNetworkPlug(BaseAmphoraTask):
         for port in ports:
             self.amphora_driver.post_network_plug(amphora, port)
             LOG.debug("post_network_plug called on compute instance "
-                      "{compute_id} for port {port_id}".format(
-                          compute_id=amphora.compute_id, port_id=port.id))
+                      "%(compute_id)s for port %(port_id)s",
+                      {"compute_id": amphora.compute_id, "port_id": port.id})
 
     def revert(self, result, amphora, *args, **kwargs):
         """Handle a failed post network plug."""
         if isinstance(result, failure.Failure):
             return
-        LOG.warn(_LW("Reverting post network plug."))
-        self.amphora_repo.update(db_apis.get_session(), id=amphora.id,
-                                 status=constants.ERROR)
+        LOG.warning(_LW("Reverting post network plug."))
+        self.task_utils.mark_amphora_status_error(amphora.id)
 
 
 class AmphoraePostNetworkPlug(BaseAmphoraTask):
@@ -216,26 +204,120 @@ class AmphoraePostNetworkPlug(BaseAmphoraTask):
         """Handle a failed post network plug."""
         if isinstance(result, failure.Failure):
             return
-        LOG.warn(_LW("Reverting post network plug."))
-        for amphora in loadbalancer.amphorae:
-            self.amphora_repo.update(db_apis.get_session(), id=amphora.id,
-                                     status=constants.ERROR)
+        LOG.warning(_LW("Reverting post network plug."))
+        for amphora in six.moves.filter(
+            lambda amp: amp.status == constants.AMPHORA_ALLOCATED,
+                loadbalancer.amphorae):
+
+            self.task_utils.mark_amphora_status_error(amphora.id)
 
 
 class AmphoraPostVIPPlug(BaseAmphoraTask):
     """Task to notify the amphora post VIP plug."""
 
-    def execute(self, loadbalancer, amphorae_network_config):
+    def execute(self, amphora, loadbalancer, amphorae_network_config):
         """Execute post_vip_routine."""
         self.amphora_driver.post_vip_plug(
-            loadbalancer, amphorae_network_config)
-        LOG.debug("Notfied amphora of vip plug")
+            amphora, loadbalancer, amphorae_network_config)
+        LOG.debug("Notified amphora of vip plug")
+
+    def revert(self, result, amphora, loadbalancer, *args, **kwargs):
+        """Handle a failed amphora vip plug notification."""
+        if isinstance(result, failure.Failure):
+            return
+        LOG.warning(_LW("Reverting post vip plug."))
+        self.task_utils.mark_amphora_status_error(amphora.id)
+        self.task_utils.mark_loadbalancer_prov_status_error(loadbalancer.id)
+
+
+class AmphoraePostVIPPlug(BaseAmphoraTask):
+    """Task to notify the amphorae post VIP plug."""
+
+    def execute(self, loadbalancer, amphorae_network_config):
+        """Execute post_vip_plug across the amphorae."""
+        amp_post_vip_plug = AmphoraPostVIPPlug()
+        for amphora in loadbalancer.amphorae:
+            amp_post_vip_plug.execute(amphora,
+                                      loadbalancer,
+                                      amphorae_network_config)
 
     def revert(self, result, loadbalancer, *args, **kwargs):
         """Handle a failed amphora vip plug notification."""
         if isinstance(result, failure.Failure):
             return
-        LOG.warn(_LW("Reverting post vip plug."))
-        self.loadbalancer_repo.update(db_apis.get_session(),
-                                      id=loadbalancer.id,
-                                      status=constants.ERROR)
+        LOG.warning(_LW("Reverting amphorae post vip plug."))
+        self.task_utils.mark_loadbalancer_prov_status_error(loadbalancer.id)
+
+
+class AmphoraCertUpload(BaseAmphoraTask):
+    """Upload a certificate to the amphora."""
+
+    def execute(self, amphora, server_pem):
+        """Execute cert_update_amphora routine."""
+        LOG.debug("Upload cert in amphora REST driver")
+        self.amphora_driver.upload_cert_amp(amphora, server_pem)
+
+
+class AmphoraUpdateVRRPInterface(BaseAmphoraTask):
+    """Task to get and update the VRRP interface device name from amphora."""
+
+    def execute(self, loadbalancer):
+        """Execute post_vip_routine."""
+        amps = []
+        for amp in six.moves.filter(
+            lambda amp: amp.status == constants.AMPHORA_ALLOCATED,
+                loadbalancer.amphorae):
+                    # Currently this is supported only with REST Driver
+                    interface = self.amphora_driver.get_vrrp_interface(amp)
+                    self.amphora_repo.update(db_apis.get_session(), amp.id,
+                                             vrrp_interface=interface)
+                    amps.append(self.amphora_repo.get(db_apis.get_session(),
+                                                      id=amp.id))
+        loadbalancer.amphorae = amps
+        return loadbalancer
+
+    def revert(self, result, loadbalancer, *args, **kwargs):
+        """Handle a failed amphora vip plug notification."""
+        if isinstance(result, failure.Failure):
+            return
+        LOG.warning(_LW("Reverting Get Amphora VRRP Interface."))
+        for amp in six.moves.filter(
+            lambda amp: amp.status == constants.AMPHORA_ALLOCATED,
+                loadbalancer.amphorae):
+
+            try:
+                self.amphora_repo.update(db_apis.get_session(), amp.id,
+                                         vrrp_interface=None)
+            except Exception as e:
+                LOG.error(_LE("Failed to update amphora %(amp)s "
+                              "VRRP interface to None due to: "
+                              "%(except)s"), {'amp': amp.id,
+                                              'except': e})
+
+
+class AmphoraVRRPUpdate(BaseAmphoraTask):
+    """Task to update the VRRP configuration of the loadbalancer amphorae."""
+
+    def execute(self, loadbalancer):
+        """Execute update_vrrp_conf."""
+        self.amphora_driver.update_vrrp_conf(loadbalancer)
+        LOG.debug("Uploaded VRRP configuration of loadbalancer %s amphorae",
+                  loadbalancer.id)
+
+
+class AmphoraVRRPStop(BaseAmphoraTask):
+    """Task to stop keepalived of all amphorae of a LB."""
+
+    def execute(self, loadbalancer):
+        self.amphora_driver.stop_vrrp_service(loadbalancer)
+        LOG.debug("Stopped VRRP of loadbalancer % amphorae",
+                  loadbalancer.id)
+
+
+class AmphoraVRRPStart(BaseAmphoraTask):
+    """Task to start keepalived of all amphorae of a LB."""
+
+    def execute(self, loadbalancer):
+        self.amphora_driver.start_vrrp_service(loadbalancer)
+        LOG.debug("Started VRRP of loadbalancer %s amphorae",
+                  loadbalancer.id)
