@@ -15,6 +15,7 @@ import hashlib
 import json
 import os
 import random
+import socket
 import stat
 import subprocess
 
@@ -22,10 +23,10 @@ import mock
 import netifaces
 from oslo_config import cfg
 import six
-import six.moves.builtins as builtins
 
 from octavia.amphorae.backends.agent import api_server
 from octavia.amphorae.backends.agent.api_server import certificate_update
+from octavia.amphorae.backends.agent.api_server import listener
 from octavia.amphorae.backends.agent.api_server import server
 from octavia.amphorae.backends.agent.api_server import util
 from octavia.common import constants as consts
@@ -54,10 +55,10 @@ class ServerTestCase(base.TestCase):
                      mock_makedirs, mock_exists):
 
         mock_exists.return_value = True
-        m = mock.mock_open()
 
         flags = os.O_WRONLY | os.O_CREAT | os.O_TRUNC
         file_name = '/var/lib/octavia/123/haproxy.cfg.new'
+        m = self.useFixture(test_utils.OpenFixture(file_name)).mock_open
 
         # happy case init file exists
         with mock.patch('os.open') as mock_open, mock.patch.object(
@@ -85,7 +86,7 @@ class ServerTestCase(base.TestCase):
                 '/var/lib/octavia/123/haproxy.cfg')
 
         # exception writing
-        m = mock.mock_open()
+        m = self.useFixture(test_utils.OpenFixture(file_name)).mock_open
         m.side_effect = IOError()  # open crashes
         with mock.patch('os.open'), mock.patch.object(os, 'fdopen', m):
             rv = self.app.put('/' + api_server.VERSION +
@@ -95,7 +96,8 @@ class ServerTestCase(base.TestCase):
 
         # check if files get created
         mock_exists.return_value = False
-        m = mock.mock_open()
+        init_path = '/etc/init/haproxy-123.conf'
+        m = self.useFixture(test_utils.OpenFixture(init_path)).mock_open
 
         # happy case init file exists
         with mock.patch('os.open') as mock_open, mock.patch.object(
@@ -171,7 +173,7 @@ class ServerTestCase(base.TestCase):
         self.assertEqual(
             {'message': 'OK',
              'details': 'Configuration file is valid\nhaproxy daemon for'
-                        + ' 123 started'},
+                        ' 123 started'},
             json.loads(rv.data.decode('utf-8')))
         mock_subprocess.assert_called_with(
             ['/usr/sbin/service', 'haproxy-123', 'start'], stderr=-2)
@@ -278,22 +280,22 @@ class ServerTestCase(base.TestCase):
         self.assertEqual(404, rv.status_code)
 
         mock_exists.side_effect = [True]
-        m = mock.mock_open(read_data=CONTENT)
+        path = util.config_path('123')
+        self.useFixture(test_utils.OpenFixture(path, CONTENT))
 
-        with mock.patch.object(builtins, 'open', m):
-            rv = self.app.get('/' + api_server.VERSION +
-                              '/listeners/123/haproxy')
-            self.assertEqual(200, rv.status_code)
-            self.assertEqual(six.b(CONTENT), rv.data)
-            self.assertEqual('text/plain; charset=utf-8',
-                             rv.headers['Content-Type'])
+        rv = self.app.get('/' + api_server.VERSION +
+                          '/listeners/123/haproxy')
+        self.assertEqual(200, rv.status_code)
+        self.assertEqual(six.b(CONTENT), rv.data)
+        self.assertEqual('text/plain; charset=utf-8',
+                         rv.headers['Content-Type'])
 
     @mock.patch('octavia.amphorae.backends.agent.api_server.util.'
-                + 'get_listeners')
+                'get_listeners')
     @mock.patch('octavia.amphorae.backends.agent.api_server.listener.'
-                + '_check_listener_status')
+                '_check_listener_status')
     @mock.patch('octavia.amphorae.backends.agent.api_server.listener.'
-                + '_parse_haproxy_file')
+                '_parse_haproxy_file')
     def test_get_all_listeners(self, mock_parse, mock_status, mock_listener):
         # no listeners
         mock_listener.side_effect = [[]]
@@ -326,9 +328,9 @@ class ServerTestCase(base.TestCase):
             json.loads(rv.data.decode('utf-8')))
 
     @mock.patch('octavia.amphorae.backends.agent.api_server.listener.'
-                + '_check_listener_status')
+                '_check_listener_status')
     @mock.patch('octavia.amphorae.backends.agent.api_server.listener.'
-                + '_parse_haproxy_file')
+                '_parse_haproxy_file')
     @mock.patch('octavia.amphorae.backends.utils.haproxy_query.HAProxyQuery')
     @mock.patch('os.path.exists')
     def test_get_listener(self, mock_exists, mock_query, mock_parse,
@@ -430,12 +432,12 @@ class ServerTestCase(base.TestCase):
                           data='TestTest')
         self.assertEqual(400, rv.status_code)
 
-        m = mock.mock_open(read_data=CONTENT)
         mock_exists.return_value = True
         mock_exists.side_effect = None
-        with mock.patch.object(builtins, 'open', m):
-            rv = self.app.get('/' + api_server.VERSION +
-                              '/listeners/123/certificates/test.pem')
+        path = listener._cert_file_path('123', 'test.pem')
+        self.useFixture(test_utils.OpenFixture(path, CONTENT))
+        rv = self.app.get('/' + api_server.VERSION +
+                          '/listeners/123/certificates/test.pem')
         self.assertEqual(200, rv.status_code)
         self.assertEqual(dict(md5sum=hashlib.md5(six.b(CONTENT)).hexdigest()),
                          json.loads(rv.data.decode('utf-8')))
@@ -450,7 +452,8 @@ class ServerTestCase(base.TestCase):
         self.assertEqual(400, rv.status_code)
 
         mock_exists.return_value = True
-        m = mock.mock_open()
+        path = listener._cert_file_path('123', 'test.pem')
+        m = self.useFixture(test_utils.OpenFixture(path)).mock_open
 
         with mock.patch('os.open'), mock.patch.object(os, 'fdopen', m):
             rv = self.app.put('/' + api_server.VERSION +
@@ -462,7 +465,7 @@ class ServerTestCase(base.TestCase):
             handle.write.assert_called_once_with(six.b('TestTest'))
 
         mock_exists.return_value = False
-        m = mock.mock_open()
+        m = self.useFixture(test_utils.OpenFixture(path)).mock_open
 
         with mock.patch('os.open'), mock.patch.object(os, 'fdopen', m):
             rv = self.app.put('/' + api_server.VERSION +
@@ -476,7 +479,8 @@ class ServerTestCase(base.TestCase):
 
     def test_upload_server_certificate(self):
         certificate_update.BUFFER = 5  # test the while loop
-        m = mock.mock_open()
+        path = '/etc/octavia/certs/server.pem'
+        m = self.useFixture(test_utils.OpenFixture(path)).mock_open
         with mock.patch('os.open'), mock.patch.object(os, 'fdopen', m):
             rv = self.app.put('/' + api_server.VERSION +
                               '/certificate',
@@ -492,12 +496,14 @@ class ServerTestCase(base.TestCase):
     @mock.patch('pyroute2.IPRoute')
     @mock.patch('pyroute2.NetNS')
     @mock.patch('subprocess.check_output')
-    def test_plug_network(self, mock_check_output, mock_netns,
-                          mock_pyroute2, mock_ifaddress,
-                          mock_interfaces):
+    @mock.patch('octavia.amphorae.backends.agent.api_server.'
+                'plug._netns_interface_exists')
+    def test_plug_network(self, mock_int_exists, mock_check_output, mock_netns,
+                          mock_pyroute2, mock_ifaddress, mock_interfaces):
         port_info = {'mac_address': '123'}
         test_int_num = random.randint(0, 9999)
 
+        mock_int_exists.return_value = False
         netns_handle = mock_netns.return_value.__enter__.return_value
         netns_handle.get_links.return_value = [0] * test_int_num
 
@@ -536,6 +542,7 @@ class ServerTestCase(base.TestCase):
         with mock.patch('os.open') as mock_open, mock.patch.object(
                 os, 'fdopen', m) as mock_fdopen:
             mock_open.return_value = 123
+
             rv = self.app.post('/' + api_server.VERSION + "/plug/network",
                                content_type='application/json',
                                data=json.dumps(port_info))
@@ -553,8 +560,10 @@ class ServerTestCase(base.TestCase):
             handle = m()
             handle.write.assert_any_call(
                 '\n# Generated by Octavia agent\n'
-                'auto eth' + test_int_num + ' eth' + test_int_num + ':0\n'
-                'iface eth' + test_int_num + ' inet dhcp')
+                'auto eth' + test_int_num + '\n'
+                'iface eth' + test_int_num + ' inet dhcp\n'
+                'auto eth' + test_int_num + ':0\n'
+                'iface eth' + test_int_num + ':0 inet6 auto\n')
             mock_check_output.assert_called_with(
                 ['ip', 'netns', 'exec', 'amphora-haproxy', 'ifup',
                  'eth' + test_int_num], stderr=-2)
@@ -566,7 +575,8 @@ class ServerTestCase(base.TestCase):
         mock_check_output.side_effect = [subprocess.CalledProcessError(
             7, 'test', RANDOM_ERROR), subprocess.CalledProcessError(
             7, 'test', RANDOM_ERROR)]
-        m = mock.mock_open()
+
+        m = self.useFixture(test_utils.OpenFixture(file_name)).mock_open
         with mock.patch('os.open'), mock.patch.object(os, 'fdopen', m):
             rv = self.app.post('/' + api_server.VERSION + "/plug/network",
                                content_type='application/json',
@@ -577,6 +587,78 @@ class ServerTestCase(base.TestCase):
                  'message': 'Error plugging network'},
                 json.loads(rv.data.decode('utf-8')))
 
+    @mock.patch('netifaces.interfaces')
+    @mock.patch('netifaces.ifaddresses')
+    @mock.patch('pyroute2.IPRoute')
+    @mock.patch('pyroute2.NetNS')
+    @mock.patch('subprocess.check_output')
+    def test_plug_network_host_routes(self, mock_check_output, mock_netns,
+                                      mock_pyroute2, mock_ifaddress,
+                                      mock_interfaces):
+        SUBNET_CIDR = '192.0.2.0/24'
+        BROADCAST = '192.0.2.255'
+        NETMASK = '255.255.255.0'
+        IP = '192.0.1.5'
+        MAC = '123'
+        DEST1 = '198.51.100.0/24'
+        DEST2 = '203.0.113.0/24'
+        NEXTHOP = '192.0.2.1'
+
+        netns_handle = mock_netns.return_value.__enter__.return_value
+        netns_handle.get_links.return_value = [{
+            'attrs': [['IFLA_IFNAME', consts.NETNS_PRIMARY_INTERFACE]]}]
+
+        port_info = {'mac_address': MAC, 'fixed_ips': [
+            {'ip_address': IP, 'subnet_cidr': SUBNET_CIDR,
+             'host_routes': [{'destination': DEST1, 'nexthop': NEXTHOP},
+                             {'destination': DEST2, 'nexthop': NEXTHOP}]}]}
+        mock_interfaces.side_effect = [['blah']]
+        mock_ifaddress.side_effect = [[netifaces.AF_LINK],
+                                      {netifaces.AF_LINK: [{'addr': '123'}]}]
+
+        flags = os.O_WRONLY | os.O_CREAT | os.O_TRUNC
+        mode = stat.S_IRUSR | stat.S_IWUSR | stat.S_IRGRP | stat.S_IROTH
+        file_name = '/etc/netns/{0}/network/interfaces.d/{1}.cfg'.format(
+            consts.AMPHORA_NAMESPACE, consts.NETNS_PRIMARY_INTERFACE)
+        m = self.useFixture(test_utils.OpenFixture(file_name)).mock_open
+        with mock.patch('os.open') as mock_open, mock.patch.object(
+                os, 'fdopen', m) as mock_fdopen:
+            mock_open.return_value = 123
+            rv = self.app.post('/' + api_server.VERSION + "/plug/network",
+                               content_type='application/json',
+                               data=json.dumps(port_info))
+            self.assertEqual(202, rv.status_code)
+
+            mock_open.assert_any_call(file_name, flags, mode)
+            mock_fdopen.assert_any_call(123, 'w')
+
+            plug_inf_file = '/var/lib/octavia/plugged_interfaces'
+            flags = os.O_RDWR | os.O_CREAT
+            mode = stat.S_IRUSR | stat.S_IWUSR | stat.S_IRGRP | stat.S_IROTH
+            mock_open.assert_any_call(plug_inf_file, flags, mode)
+            mock_fdopen.assert_any_call(123, 'r+')
+
+            handle = m()
+            handle.write.assert_any_call(
+                '\n\n# Generated by Octavia agent\n'
+                'auto ' + consts.NETNS_PRIMARY_INTERFACE +
+                '\niface ' + consts.NETNS_PRIMARY_INTERFACE +
+                ' inet static\n' +
+                'address ' + IP + '\nbroadcast ' + BROADCAST + '\n' +
+                'netmask ' + NETMASK + '\n' +
+                'up route add -net ' + DEST1 + ' gw ' + NEXTHOP +
+                ' dev ' + consts.NETNS_PRIMARY_INTERFACE + '\n'
+                'down route del -net ' + DEST1 + ' gw ' + NEXTHOP +
+                ' dev ' + consts.NETNS_PRIMARY_INTERFACE + '\n'
+                'up route add -net ' + DEST2 + ' gw ' + NEXTHOP +
+                ' dev ' + consts.NETNS_PRIMARY_INTERFACE + '\n'
+                'down route del -net ' + DEST2 + ' gw ' + NEXTHOP +
+                ' dev ' + consts.NETNS_PRIMARY_INTERFACE + '\n'
+            )
+            mock_check_output.assert_called_with(
+                ['ip', 'netns', 'exec', consts.AMPHORA_NAMESPACE,
+                 'ifup', consts.NETNS_PRIMARY_INTERFACE], stderr=-2)
+
     @mock.patch('netifaces.interfaces')
     @mock.patch('netifaces.ifaddresses')
     @mock.patch('pyroute2.IPRoute')
@@ -585,15 +667,17 @@ class ServerTestCase(base.TestCase):
     @mock.patch('subprocess.check_output')
     @mock.patch('shutil.copytree')
     @mock.patch('os.makedirs')
-    def test_plug_VIP(self, mock_makedirs, mock_copytree, mock_check_output,
-                      mock_netns, mock_netns_create, mock_pyroute2,
-                      mock_ifaddress, mock_interfaces):
+    def test_plug_vip4(self, mock_makedirs, mock_copytree, mock_check_output,
+                       mock_netns, mock_netns_create, mock_pyroute2,
+                       mock_ifaddress, mock_interfaces):
 
-        subnet_info = {'subnet_cidr': '10.0.0.0/24',
-                       'gateway': '10.0.0.1',
-                       'mac_address': '123'}
+        subnet_info = {
+            'subnet_cidr': '203.0.113.0/24',
+            'gateway': '203.0.113.1',
+            'mac_address': '123'
+        }
 
-        # malformated ip
+        # malformed ip
         rv = self.app.post('/' + api_server.VERSION + '/plug/vip/error',
                            data=json.dumps(subnet_info),
                            content_type='application/json')
@@ -622,15 +706,86 @@ class ServerTestCase(base.TestCase):
         self.assertEqual(dict(details="No suitable network interface found"),
                          json.loads(rv.data.decode('utf-8')))
 
-        # One Interface down, Happy Path
+        # Happy Path IPv4, with VRRP_IP and host route
+        full_subnet_info = {
+            'subnet_cidr': '203.0.113.0/24',
+            'gateway': '203.0.113.1',
+            'mac_address': '123',
+            'vrrp_ip': '203.0.113.4',
+            'host_routes': [{'destination': '203.0.114.0/24',
+                             'nexthop': '203.0.113.5'},
+                            {'destination': '203.0.115.0/24',
+                             'nexthop': '203.0.113.5'}]
+        }
+
         mock_interfaces.side_effect = [['blah']]
         mock_ifaddress.side_effect = [[netifaces.AF_LINK],
                                       {netifaces.AF_LINK: [{'addr': '123'}]}]
 
         flags = os.O_WRONLY | os.O_CREAT | os.O_TRUNC
         mode = stat.S_IRUSR | stat.S_IWUSR | stat.S_IRGRP | stat.S_IROTH
-        file_name = '/etc/netns/{0}/network/interfaces.d/blah.cfg'.format(
-            consts.AMPHORA_NAMESPACE)
+        file_name = ('/etc/netns/{netns}/network/interfaces.d/'
+                     '{netns_int}.cfg'.format(
+                         netns=consts.AMPHORA_NAMESPACE,
+                         netns_int=consts.NETNS_PRIMARY_INTERFACE))
+        m = self.useFixture(test_utils.OpenFixture(file_name)).mock_open
+
+        with mock.patch('os.open') as mock_open, mock.patch.object(
+                os, 'fdopen', m) as mock_fdopen:
+            mock_open.return_value = 123
+            rv = self.app.post('/' + api_server.VERSION +
+                               "/plug/vip/203.0.113.2",
+                               content_type='application/json',
+                               data=json.dumps(full_subnet_info))
+            self.assertEqual(202, rv.status_code)
+            mock_open.assert_any_call(file_name, flags, mode)
+            mock_fdopen.assert_any_call(123, 'w')
+
+            plug_inf_file = '/var/lib/octavia/plugged_interfaces'
+            flags = os.O_RDWR | os.O_CREAT
+            mode = stat.S_IRUSR | stat.S_IWUSR | stat.S_IRGRP | stat.S_IROTH
+            mock_open.assert_any_call(plug_inf_file, flags, mode)
+            mock_fdopen.assert_any_call(123, 'r+')
+
+            handle = m()
+            handle.write.assert_any_call(
+                '\n# Generated by Octavia agent\n'
+                'auto {netns_int} {netns_int}:0\n'
+                'iface {netns_int} inet static\n'
+                'address 203.0.113.4\n'
+                'broadcast 203.0.113.255\n'
+                'netmask 255.255.255.0\n'
+                'gateway 203.0.113.1\n'
+                'up route add -net 203.0.114.0/24 gw 203.0.113.5 '
+                'dev {netns_int}\n'
+                'down route del -net 203.0.114.0/24 gw 203.0.113.5 '
+                'dev {netns_int}\n'
+                'up route add -net 203.0.115.0/24 gw 203.0.113.5 '
+                'dev {netns_int}\n'
+                'down route del -net 203.0.115.0/24 gw 203.0.113.5 '
+                'dev {netns_int}\n'
+                '\n'
+                'iface {netns_int}:0 inet static\n'
+                'address 203.0.113.2\n'
+                'broadcast 203.0.113.255\n'
+                'netmask 255.255.255.0'.format(
+                    netns_int=consts.NETNS_PRIMARY_INTERFACE))
+            mock_check_output.assert_called_with(
+                ['ip', 'netns', 'exec', consts.AMPHORA_NAMESPACE,
+                 'ifup', '{netns_int}:0'.format(
+                     netns_int=consts.NETNS_PRIMARY_INTERFACE)], stderr=-2)
+
+        # One Interface down, Happy Path IPv4
+        mock_interfaces.side_effect = [['blah']]
+        mock_ifaddress.side_effect = [[netifaces.AF_LINK],
+                                      {netifaces.AF_LINK: [{'addr': '123'}]}]
+
+        flags = os.O_WRONLY | os.O_CREAT | os.O_TRUNC
+        mode = stat.S_IRUSR | stat.S_IWUSR | stat.S_IRGRP | stat.S_IROTH
+        file_name = ('/etc/netns/{netns}/network/interfaces.d/'
+                     '{netns_int}.cfg'.format(
+                         netns=consts.AMPHORA_NAMESPACE,
+                         netns_int=consts.NETNS_PRIMARY_INTERFACE))
         m = mock.mock_open()
         with mock.patch('os.open') as mock_open, mock.patch.object(
                 os, 'fdopen', m) as mock_fdopen:
@@ -653,15 +808,17 @@ class ServerTestCase(base.TestCase):
             handle = m()
             handle.write.assert_any_call(
                 '\n# Generated by Octavia agent\n'
-                'auto blah blah:0\n'
-                'iface blah inet dhcp\n'
-                'iface blah:0 inet static\n'
+                'auto {netns_int} {netns_int}:0\n'
+                'iface {netns_int} inet dhcp\n\n'
+                'iface {netns_int}:0 inet static\n'
                 'address 203.0.113.2\n'
                 'broadcast 203.0.113.255\n'
-                'netmask 255.255.255.0')
+                'netmask 255.255.255.0'.format(
+                    netns_int=consts.NETNS_PRIMARY_INTERFACE))
             mock_check_output.assert_called_with(
                 ['ip', 'netns', 'exec', 'amphora-haproxy', 'ifup',
-                 'blah:0'], stderr=-2)
+                 '{netns_int}:0'.format(
+                     netns_int=consts.NETNS_PRIMARY_INTERFACE)], stderr=-2)
 
         mock_interfaces.side_effect = [['blah']]
         mock_ifaddress.side_effect = [[netifaces.AF_LINK],
@@ -671,6 +828,7 @@ class ServerTestCase(base.TestCase):
             subprocess.CalledProcessError(
                 7, 'test', RANDOM_ERROR), subprocess.CalledProcessError(
                 7, 'test', RANDOM_ERROR)]
+
         m = mock.mock_open()
         with mock.patch('os.open'), mock.patch.object(os, 'fdopen', m):
             rv = self.app.post('/' + api_server.VERSION +
@@ -683,6 +841,186 @@ class ServerTestCase(base.TestCase):
                  'message': 'Error plugging VIP'},
                 json.loads(rv.data.decode('utf-8')))
 
+    @mock.patch('netifaces.interfaces')
+    @mock.patch('netifaces.ifaddresses')
+    @mock.patch('pyroute2.IPRoute')
+    @mock.patch('pyroute2.netns.create')
+    @mock.patch('pyroute2.NetNS')
+    @mock.patch('subprocess.check_output')
+    @mock.patch('shutil.copytree')
+    @mock.patch('os.makedirs')
+    def test_plug_vip6(self, mock_makedirs, mock_copytree, mock_check_output,
+                       mock_netns, mock_netns_create, mock_pyroute2,
+                       mock_ifaddress, mock_interfaces):
+
+        subnet_info = {
+            'subnet_cidr': '2001:db8::/32',
+            'gateway': '2001:db8::1',
+            'mac_address': '123'
+        }
+
+        # malformed ip
+        rv = self.app.post('/' + api_server.VERSION + '/plug/vip/error',
+                           data=json.dumps(subnet_info),
+                           content_type='application/json')
+        self.assertEqual(400, rv.status_code)
+
+        # No subnet info
+        rv = self.app.post('/' + api_server.VERSION + '/plug/vip/error')
+        self.assertEqual(400, rv.status_code)
+
+        # No interface at all
+        mock_interfaces.side_effect = [[]]
+        rv = self.app.post('/' + api_server.VERSION + "/plug/vip/2001:db8::2",
+                           content_type='application/json',
+                           data=json.dumps(subnet_info))
+        self.assertEqual(404, rv.status_code)
+        self.assertEqual(dict(details="No suitable network interface found"),
+                         json.loads(rv.data.decode('utf-8')))
+
+        # Two interfaces down
+        mock_interfaces.side_effect = [['blah', 'blah2']]
+        mock_ifaddress.side_effect = [['blabla'], ['blabla']]
+        rv = self.app.post('/' + api_server.VERSION + "/plug/vip/2001:db8::2",
+                           content_type='application/json',
+                           data=json.dumps(subnet_info))
+        self.assertEqual(404, rv.status_code)
+        self.assertEqual(dict(details="No suitable network interface found"),
+                         json.loads(rv.data.decode('utf-8')))
+
+        # Happy Path IPv6, with VRRP_IP and host route
+        full_subnet_info = {
+            'subnet_cidr': '2001:db8::/32',
+            'gateway': '2001:db8::1',
+            'mac_address': '123',
+            'vrrp_ip': '2001:db8::4',
+            'host_routes': [{'destination': '2001:db9::/32',
+                             'nexthop': '2001:db8::5'},
+                            {'destination': '2001:db9::/32',
+                             'nexthop': '2001:db8::5'}]
+        }
+
+        mock_interfaces.side_effect = [['blah']]
+        mock_ifaddress.side_effect = [[netifaces.AF_LINK],
+                                      {netifaces.AF_LINK: [{'addr': '123'}]}]
+
+        flags = os.O_WRONLY | os.O_CREAT | os.O_TRUNC
+        mode = stat.S_IRUSR | stat.S_IWUSR | stat.S_IRGRP | stat.S_IROTH
+        file_name = ('/etc/netns/{netns}/network/interfaces.d/'
+                     '{netns_int}.cfg'.format(
+                         netns=consts.AMPHORA_NAMESPACE,
+                         netns_int=consts.NETNS_PRIMARY_INTERFACE))
+        m = self.useFixture(test_utils.OpenFixture(file_name)).mock_open
+
+        with mock.patch('os.open') as mock_open, mock.patch.object(
+                os, 'fdopen', m) as mock_fdopen:
+            mock_open.return_value = 123
+            rv = self.app.post('/' + api_server.VERSION +
+                               "/plug/vip/2001:db8::2",
+                               content_type='application/json',
+                               data=json.dumps(full_subnet_info))
+            self.assertEqual(202, rv.status_code)
+            mock_open.assert_any_call(file_name, flags, mode)
+            mock_fdopen.assert_any_call(123, 'w')
+
+            plug_inf_file = '/var/lib/octavia/plugged_interfaces'
+            flags = os.O_RDWR | os.O_CREAT
+            mode = stat.S_IRUSR | stat.S_IWUSR | stat.S_IRGRP | stat.S_IROTH
+            mock_open.assert_any_call(plug_inf_file, flags, mode)
+            mock_fdopen.assert_any_call(123, 'r+')
+            handle = m()
+            handle.write.assert_any_call(
+                '\n# Generated by Octavia agent\n'
+                'auto {netns_int} {netns_int}:0\n'
+                'iface {netns_int} inet6 static\n'
+                'address 2001:db8::4\n'
+                'broadcast 2001:0db8:ffff:ffff:ffff:ffff:ffff:ffff\n'
+                'netmask 32\n'
+                'gateway 2001:db8::1\n'
+                'up route add -net 2001:db9::/32 gw 2001:db8::5 '
+                'dev {netns_int}\n'
+                'down route del -net 2001:db9::/32 gw 2001:db8::5 '
+                'dev {netns_int}\n'
+                'up route add -net 2001:db9::/32 gw 2001:db8::5 '
+                'dev {netns_int}\n'
+                'down route del -net 2001:db9::/32 gw 2001:db8::5 '
+                'dev {netns_int}\n'
+                '\n'
+                'iface {netns_int}:0 inet6 static\n'
+                'address 2001:0db8:0000:0000:0000:0000:0000:0002\n'
+                'broadcast 2001:0db8:ffff:ffff:ffff:ffff:ffff:ffff\n'
+                'netmask 32'.format(netns_int=consts.NETNS_PRIMARY_INTERFACE))
+            mock_check_output.assert_called_with(
+                ['ip', 'netns', 'exec', consts.AMPHORA_NAMESPACE,
+                 'ifup', '{netns_int}:0'.format(
+                     netns_int=consts.NETNS_PRIMARY_INTERFACE)], stderr=-2)
+
+        # One Interface down, Happy Path IPv6
+        mock_interfaces.side_effect = [['blah']]
+        mock_ifaddress.side_effect = [[netifaces.AF_LINK],
+                                      {netifaces.AF_LINK: [{'addr': '123'}]}]
+
+        flags = os.O_WRONLY | os.O_CREAT | os.O_TRUNC
+        mode = stat.S_IRUSR | stat.S_IWUSR | stat.S_IRGRP | stat.S_IROTH
+        file_name = ('/etc/netns/{netns}/network/interfaces.d/'
+                     '{netns_int}.cfg'.format(
+                         netns=consts.AMPHORA_NAMESPACE,
+                         netns_int=consts.NETNS_PRIMARY_INTERFACE))
+        m = mock.mock_open()
+        with mock.patch('os.open') as mock_open, mock.patch.object(
+                os, 'fdopen', m) as mock_fdopen:
+            mock_open.return_value = 123
+            rv = self.app.post('/' + api_server.VERSION +
+                               "/plug/vip/2001:db8::2",
+                               content_type='application/json',
+                               data=json.dumps(subnet_info))
+            self.assertEqual(202, rv.status_code)
+
+            mock_open.assert_any_call(file_name, flags, mode)
+            mock_fdopen.assert_any_call(123, 'w')
+
+            plug_inf_file = '/var/lib/octavia/plugged_interfaces'
+            flags = os.O_RDWR | os.O_CREAT
+            mode = stat.S_IRUSR | stat.S_IWUSR | stat.S_IRGRP | stat.S_IROTH
+            mock_open.assert_any_call(plug_inf_file, flags, mode)
+            mock_fdopen.assert_any_call(123, 'r+')
+
+            handle = m()
+            handle.write.assert_any_call(
+                '\n# Generated by Octavia agent\n'
+                'auto {netns_int} {netns_int}:0\n'
+                'iface {netns_int} inet6 auto\n\n'
+                'iface {netns_int}:0 inet6 static\n'
+                'address 2001:0db8:0000:0000:0000:0000:0000:0002\n'
+                'broadcast 2001:0db8:ffff:ffff:ffff:ffff:ffff:ffff\n'
+                'netmask 32'.format(
+                    netns_int=consts.NETNS_PRIMARY_INTERFACE))
+            mock_check_output.assert_called_with(
+                ['ip', 'netns', 'exec', 'amphora-haproxy', 'ifup',
+                 '{netns_int}:0'.format(
+                     netns_int=consts.NETNS_PRIMARY_INTERFACE)], stderr=-2)
+
+        mock_interfaces.side_effect = [['blah']]
+        mock_ifaddress.side_effect = [[netifaces.AF_LINK],
+                                      {netifaces.AF_LINK: [{'addr': '123'}]}]
+        mock_check_output.side_effect = [
+            'unplug1',
+            subprocess.CalledProcessError(
+                7, 'test', RANDOM_ERROR), subprocess.CalledProcessError(
+                7, 'test', RANDOM_ERROR)]
+
+        m = mock.mock_open()
+        with mock.patch('os.open'), mock.patch.object(os, 'fdopen', m):
+            rv = self.app.post('/' + api_server.VERSION +
+                               "/plug/vip/2001:db8::2",
+                               content_type='application/json',
+                               data=json.dumps(subnet_info))
+            self.assertEqual(500, rv.status_code)
+            self.assertEqual(
+                {'details': RANDOM_ERROR,
+                 'message': 'Error plugging VIP'},
+                json.loads(rv.data.decode('utf-8')))
+
     @mock.patch('pyroute2.NetNS')
     def test_get_interface(self, mock_netns):
 
@@ -692,7 +1030,7 @@ class ServerTestCase(base.TestCase):
 
         # Happy path
         netns_handle.get_addr.return_value = [{
-            'index': 3, 'family': 2,
+            'index': 3, 'family': socket.AF_INET,
             'attrs': [['IFA_ADDRESS', '203.0.113.2']]}]
         netns_handle.get_links.return_value = [{
             'attrs': [['IFLA_IFNAME', 'eth0']]}]
@@ -703,7 +1041,7 @@ class ServerTestCase(base.TestCase):
 
         # Happy path with IPv6 address normalization
         netns_handle.get_addr.return_value = [{
-            'index': 3, 'family': 10,
+            'index': 3, 'family': socket.AF_INET6,
             'attrs': [['IFA_ADDRESS',
                        '0000:0000:0000:0000:0000:0000:0000:0001']]}]
         netns_handle.get_links.return_value = [{
@@ -737,7 +1075,7 @@ class ServerTestCase(base.TestCase):
 
         mock_exists.return_value = True
         cfg_path = util.keepalived_cfg_path()
-        m = mock.mock_open()
+        m = self.useFixture(test_utils.OpenFixture(cfg_path)).mock_open
 
         with mock.patch('os.open') as mock_open, mock.patch.object(
                 os, 'fdopen', m) as mock_fdopen:
@@ -752,7 +1090,7 @@ class ServerTestCase(base.TestCase):
 
         mock_exists.return_value = False
         script_path = util.keepalived_check_script_path()
-        m = mock.mock_open()
+        m = self.useFixture(test_utils.OpenFixture(script_path)).mock_open
 
         with mock.patch('os.open') as mock_open, mock.patch.object(
                 os, 'fdopen', m) as mock_fdopen:
