@@ -47,7 +47,7 @@ class TestHaproxyCfg(base.TestCase):
               "    timeout check 31\n"
               "    option httpchk GET /index.html\n"
               "    http-check expect rstatus 418\n"
-              "    option forwardfor\n"
+              "    fullconn 98\n"
               "    server sample_member_id_1 10.0.0.99:82 "
               "weight 13 check inter 30s fall 3 rise 2 "
               "cookie sample_member_id_1\n"
@@ -58,6 +58,7 @@ class TestHaproxyCfg(base.TestCase):
             certificate='imaCert1', private_key='imaPrivateKey1',
             primary_cn='FakeCN')
         rendered_obj = self.jinja_cfg.render_loadbalancer_obj(
+            sample_configs.sample_amphora_tuple(),
             sample_configs.sample_listener_tuple(proto='TERMINATED_HTTPS',
                                                  tls=True, sni=True),
             tls_tupe)
@@ -83,7 +84,7 @@ class TestHaproxyCfg(base.TestCase):
               "    timeout check 31\n"
               "    option httpchk GET /index.html\n"
               "    http-check expect rstatus 418\n"
-              "    option forwardfor\n"
+              "    fullconn 98\n"
               "    server sample_member_id_1 10.0.0.99:82 "
               "weight 13 check inter 30s fall 3 rise 2 "
               "cookie sample_member_id_1\n"
@@ -91,6 +92,7 @@ class TestHaproxyCfg(base.TestCase):
               "weight 13 check inter 30s fall 3 rise 2 "
               "cookie sample_member_id_2\n\n")
         rendered_obj = self.jinja_cfg.render_loadbalancer_obj(
+            sample_configs.sample_amphora_tuple(),
             sample_configs.sample_listener_tuple(
                 proto='TERMINATED_HTTPS', tls=True),
             tls_cert=sample_configs.sample_tls_container_tuple(
@@ -110,7 +112,7 @@ class TestHaproxyCfg(base.TestCase):
               "    timeout check 31\n"
               "    option httpchk GET /index.html\n"
               "    http-check expect rstatus 418\n"
-              "    option forwardfor\n"
+              "    fullconn 98\n"
               "    server sample_member_id_1 10.0.0.99:82 "
               "weight 13 check inter 30s fall 3 rise 2 "
               "cookie sample_member_id_1\n"
@@ -118,6 +120,7 @@ class TestHaproxyCfg(base.TestCase):
               "weight 13 check inter 30s fall 3 rise 2 "
               "cookie sample_member_id_2\n\n")
         rendered_obj = self.jinja_cfg.render_loadbalancer_obj(
+            sample_configs.sample_amphora_tuple(),
             sample_configs.sample_listener_tuple())
         self.assertEqual(
             sample_configs.sample_base_expected_config(backend=be),
@@ -138,6 +141,7 @@ class TestHaproxyCfg(base.TestCase):
               "    option httpchk GET /index.html\n"
               "    http-check expect rstatus 418\n"
               "    option ssl-hello-chk\n"
+              "    fullconn 98\n"
               "    server sample_member_id_1 10.0.0.99:82 "
               "weight 13 check inter 30s fall 3 rise 2 "
               "cookie sample_member_id_1\n"
@@ -145,6 +149,7 @@ class TestHaproxyCfg(base.TestCase):
               "weight 13 check inter 30s fall 3 rise 2 "
               "cookie sample_member_id_2\n\n")
         rendered_obj = self.jinja_cfg.render_loadbalancer_obj(
+            sample_configs.sample_amphora_tuple(),
             sample_configs.sample_listener_tuple(proto='HTTPS'))
         self.assertEqual(sample_configs.sample_base_expected_config(
             frontend=fe, backend=be), rendered_obj)
@@ -154,12 +159,13 @@ class TestHaproxyCfg(base.TestCase):
               "    mode http\n"
               "    balance roundrobin\n"
               "    cookie SRV insert indirect nocache\n"
-              "    option forwardfor\n"
+              "    fullconn 98\n"
               "    server sample_member_id_1 10.0.0.99:82 weight 13 "
               "cookie sample_member_id_1\n"
               "    server sample_member_id_2 10.0.0.98:82 weight 13 "
               "cookie sample_member_id_2\n\n")
         rendered_obj = self.jinja_cfg.render_loadbalancer_obj(
+            sample_configs.sample_amphora_tuple(),
             sample_configs.sample_listener_tuple(proto='HTTP', monitor=False))
         self.assertEqual(sample_configs.sample_base_expected_config(
             backend=be), rendered_obj)
@@ -175,11 +181,13 @@ class TestHaproxyCfg(base.TestCase):
               "    mode tcp\n"
               "    balance roundrobin\n"
               "    cookie SRV insert indirect nocache\n"
+              "    fullconn 98\n"
               "    server sample_member_id_1 10.0.0.99:82 weight 13 "
               "cookie sample_member_id_1\n"
               "    server sample_member_id_2 10.0.0.98:82 weight 13 "
               "cookie sample_member_id_2\n\n")
         rendered_obj = self.jinja_cfg.render_loadbalancer_obj(
+            sample_configs.sample_amphora_tuple(),
             sample_configs.sample_listener_tuple(proto='HTTPS', monitor=False))
         self.assertEqual(sample_configs.sample_base_expected_config(
             frontend=fe, backend=be), rendered_obj)
@@ -194,9 +202,11 @@ class TestHaproxyCfg(base.TestCase):
         be = ("backend sample_pool_id_1\n"
               "    mode tcp\n"
               "    balance roundrobin\n"
+              "    fullconn 98\n"
               "    server sample_member_id_1 10.0.0.99:82 weight 13\n"
               "    server sample_member_id_2 10.0.0.98:82 weight 13\n\n")
         rendered_obj = self.jinja_cfg.render_loadbalancer_obj(
+            sample_configs.sample_amphora_tuple(),
             sample_configs.sample_listener_tuple(proto='HTTPS', monitor=False,
                                                  persistence=False))
         self.assertEqual(sample_configs.sample_base_expected_config(
@@ -206,10 +216,11 @@ class TestHaproxyCfg(base.TestCase):
         be = ("backend sample_pool_id_1\n"
               "    mode http\n"
               "    balance roundrobin\n"
-              "    option forwardfor\n"
+              "    fullconn 98\n"
               "    server sample_member_id_1 10.0.0.99:82 weight 13\n"
               "    server sample_member_id_2 10.0.0.98:82 weight 13\n\n")
         rendered_obj = self.jinja_cfg.render_loadbalancer_obj(
+            sample_configs.sample_amphora_tuple(),
             sample_configs.sample_listener_tuple(proto='HTTP', monitor=False,
                                                  persistence=False))
         self.assertEqual(sample_configs.sample_base_expected_config(
@@ -224,12 +235,13 @@ class TestHaproxyCfg(base.TestCase):
               "    timeout check 31\n"
               "    option httpchk GET /index.html\n"
               "    http-check expect rstatus 418\n"
-              "    option forwardfor\n"
+              "    fullconn 98\n"
               "    server sample_member_id_1 10.0.0.99:82 "
               "weight 13 check inter 30s fall 3 rise 2\n"
               "    server sample_member_id_2 10.0.0.98:82 "
               "weight 13 check inter 30s fall 3 rise 2\n\n")
         rendered_obj = self.jinja_cfg.render_loadbalancer_obj(
+            sample_configs.sample_amphora_tuple(),
             sample_configs.sample_listener_tuple(
                 persistence_type='SOURCE_IP'))
         self.assertEqual(
@@ -246,12 +258,13 @@ class TestHaproxyCfg(base.TestCase):
               "    timeout check 31\n"
               "    option httpchk GET /index.html\n"
               "    http-check expect rstatus 418\n"
-              "    option forwardfor\n"
+              "    fullconn 98\n"
               "    server sample_member_id_1 10.0.0.99:82 "
               "weight 13 check inter 30s fall 3 rise 2\n"
               "    server sample_member_id_2 10.0.0.98:82 "
               "weight 13 check inter 30s fall 3 rise 2\n\n")
         rendered_obj = self.jinja_cfg.render_loadbalancer_obj(
+            sample_configs.sample_amphora_tuple(),
             sample_configs.sample_listener_tuple(
                 persistence_type='APP_COOKIE',
                 persistence_cookie='JSESSIONID'))
@@ -286,7 +299,7 @@ class TestHaproxyCfg(base.TestCase):
               "    timeout check 31\n"
               "    option httpchk GET /index.html\n"
               "    http-check expect rstatus 418\n"
-              "    option forwardfor\n"
+              "    fullconn 98\n"
               "    server sample_member_id_1 10.0.0.99:82 weight 13 check "
               "inter 30s fall 3 rise 2 cookie sample_member_id_1\n"
               "    server sample_member_id_2 10.0.0.98:82 weight 13 check "
@@ -299,14 +312,65 @@ class TestHaproxyCfg(base.TestCase):
               "    timeout check 31\n"
               "    option httpchk GET /healthmon.html\n"
               "    http-check expect rstatus 418\n"
-              "    option forwardfor\n"
+              "    fullconn 98\n"
               "    server sample_member_id_3 10.0.0.97:82 weight 13 check "
               "inter 30s fall 3 rise 2 cookie sample_member_id_3\n\n")
         rendered_obj = self.jinja_cfg.render_loadbalancer_obj(
+            sample_configs.sample_amphora_tuple(),
             sample_configs.sample_listener_tuple(l7=True))
         self.assertEqual(sample_configs.sample_base_expected_config(
             frontend=fe, backend=be), rendered_obj)
 
+    def test_render_template_http_xff(self):
+        be = ("backend sample_pool_id_1\n"
+              "    mode http\n"
+              "    balance roundrobin\n"
+              "    cookie SRV insert indirect nocache\n"
+              "    timeout check 31\n"
+              "    option httpchk GET /index.html\n"
+              "    http-check expect rstatus 418\n"
+              "    option forwardfor\n"
+              "    fullconn 98\n"
+              "    server sample_member_id_1 10.0.0.99:82 "
+              "weight 13 check inter 30s fall 3 rise 2 "
+              "cookie sample_member_id_1\n"
+              "    server sample_member_id_2 10.0.0.98:82 "
+              "weight 13 check inter 30s fall 3 rise 2 "
+              "cookie sample_member_id_2\n\n")
+        rendered_obj = self.jinja_cfg.render_loadbalancer_obj(
+            sample_configs.sample_amphora_tuple(),
+            sample_configs.sample_listener_tuple(
+                insert_headers={'X-Forwarded-For': 'true'}))
+        self.assertEqual(
+            sample_configs.sample_base_expected_config(backend=be),
+            rendered_obj)
+
+    def test_render_template_http_xff_xfport(self):
+        be = ("backend sample_pool_id_1\n"
+              "    mode http\n"
+              "    balance roundrobin\n"
+              "    cookie SRV insert indirect nocache\n"
+              "    timeout check 31\n"
+              "    option httpchk GET /index.html\n"
+              "    http-check expect rstatus 418\n"
+              "    option forwardfor\n"
+              "    http-request set-header X-Forwarded-Port %[dst_port]\n"
+              "    fullconn 98\n"
+              "    server sample_member_id_1 10.0.0.99:82 "
+              "weight 13 check inter 30s fall 3 rise 2 "
+              "cookie sample_member_id_1\n"
+              "    server sample_member_id_2 10.0.0.98:82 "
+              "weight 13 check inter 30s fall 3 rise 2 "
+              "cookie sample_member_id_2\n\n")
+        rendered_obj = self.jinja_cfg.render_loadbalancer_obj(
+            sample_configs.sample_amphora_tuple(),
+            sample_configs.sample_listener_tuple(
+                insert_headers={'X-Forwarded-For': 'true',
+                                'X-Forwarded-Port': 'true'}))
+        self.assertEqual(
+            sample_configs.sample_base_expected_config(backend=be),
+            rendered_obj)
+
     def test_transform_session_persistence(self):
         in_persistence = sample_configs.sample_session_persistence_tuple()
         ret = self.jinja_cfg._transform_session_persistence(in_persistence)
@@ -344,15 +408,22 @@ class TestHaproxyCfg(base.TestCase):
         self.assertEqual(sample_configs.RET_LISTENER_L7, ret)
 
     def test_transform_loadbalancer(self):
+        in_amphora = sample_configs.sample_amphora_tuple()
         in_listener = sample_configs.sample_listener_tuple()
         ret = self.jinja_cfg._transform_loadbalancer(
-            in_listener.load_balancer, in_listener, None)
+            in_amphora, in_listener.load_balancer, in_listener, None)
         self.assertEqual(sample_configs.RET_LB, ret)
 
+    def test_transform_amphora(self):
+        in_amphora = sample_configs.sample_amphora_tuple()
+        ret = self.jinja_cfg._transform_amphora(in_amphora)
+        self.assertEqual(sample_configs.RET_AMPHORA, ret)
+
     def test_transform_loadbalancer_with_l7(self):
+        in_amphora = sample_configs.sample_amphora_tuple()
         in_listener = sample_configs.sample_listener_tuple(l7=True)
         ret = self.jinja_cfg._transform_loadbalancer(
-            in_listener.load_balancer, in_listener, None)
+            in_amphora, in_listener.load_balancer, in_listener, None)
         self.assertEqual(sample_configs.RET_LB_L7, ret)
 
     def test_transform_l7policy(self):
